from unittest import TestCase

from kfk.config import KUBECTL_PATH
from kfk.kubectl_command_builder import Kubectl


class TestKubectl(TestCase):
    def test_get(self):
        self.assertEqual(
            Kubectl().get().build(), "{kubectl} get".format(kubectl=KUBECTL_PATH)
        )

    def test_apply(self):
        self.assertEqual(
            Kubectl().apply().build(), "{kubectl} apply".format(kubectl=KUBECTL_PATH)
        )

    def test_describe(self):
        self.assertEqual(
            Kubectl().describe().build(),
            "{kubectl} describe".format(kubectl=KUBECTL_PATH),
        )

    def test_exec(self):
        self.assertEqual(
            Kubectl()
            .exec("-it", "test-pod")
            .container("test-container")
            .namespace("test-namespace")
            .exec_command("echo 'test'")
            .build(),
            "{kubectl} exec -it test-pod -c test-container -n test-namespace -- bash -c"
            " \"echo 'test'\"".format(kubectl=KUBECTL_PATH),
        )

    def test_kafkas_all(self):
        self.assertEqual(
            Kubectl().get().kafkas().build(),
            "{kubectl} get kafkas".format(kubectl=KUBECTL_PATH),
        )

    def test_kafkas_specific(self):
        self.assertEqual(
            Kubectl().get().kafkas("my-cluster").build(),
            "{kubectl} get kafkas my-cluster".format(kubectl=KUBECTL_PATH),
        )

    def test_kafkatopics_all(self):
        self.assertEqual(
            Kubectl().get().kafkatopics().build(),
            "{kubectl} get kafkatopics".format(kubectl=KUBECTL_PATH),
        )

    def test_kafkatopics_specific(self):
        self.assertEqual(
            Kubectl().get().kafkatopics("my-topic").build(),
            "{kubectl} get kafkatopics my-topic".format(kubectl=KUBECTL_PATH),
        )

    def test_kafkausers_all(self):
        self.assertEqual(
            Kubectl().get().kafkausers().build(),
            "{kubectl} get kafkausers".format(kubectl=KUBECTL_PATH),
        )

    def test_kafkausers_specific(self):
        self.assertEqual(
            Kubectl().get().kafkausers("my-user").build(),
            "{kubectl} get kafkausers my-user".format(kubectl=KUBECTL_PATH),
        )

    def test_configmap_yaml(self):
        self.assertEqual(
            Kubectl().get().configmap("my-cluster-kafka-config").output("yaml").build(),
            "{kubectl} get configmap my-cluster-kafka-config -o yaml".format(
                kubectl=KUBECTL_PATH
            ),
        )

    def test_label(self):
        self.assertEqual(
            Kubectl().get().kafkatopics().label("app=test").build(),
            "{kubectl} get kafkatopics -l app=test".format(kubectl=KUBECTL_PATH),
        )

    def test_labels(self):
        self.assertEqual(
            Kubectl().get().kafkatopics().label("app=test, included=true").build(),
            "{kubectl} get kafkatopics -l app=test, included=true".format(
                kubectl=KUBECTL_PATH
            ),
        )

    def test_namespace(self):
        self.assertEqual(
            Kubectl().get().kafkatopics().namespace("test").build(),
            "{kubectl} get kafkatopics -n test".format(kubectl=KUBECTL_PATH),
        )

    def test_namespace_all(self):
        self.assertEqual(
            Kubectl().get().kafkatopics().namespace().build(),
            "{kubectl} get kafkatopics --all-namespaces".format(kubectl=KUBECTL_PATH),
        )

    def test_label_namespace(self):
        self.assertEqual(
            Kubectl().get().kafkatopics().label("app=test").namespace("test").build(),
            "{kubectl} get kafkatopics -l app=test -n test".format(
                kubectl=KUBECTL_PATH
            ),
        )

    def test_cluster(self):
        self.assertEqual(
            Kubectl().get().kafkas().container("test-container").build(),
            "{kubectl} get kafkas -c test-container".format(kubectl=KUBECTL_PATH),
        )

    def test_output(self):
        self.assertEqual(
            Kubectl().get().kafkas().output("yaml").build(),
            "{kubectl} get kafkas -o yaml".format(kubectl=KUBECTL_PATH),
        )



import click
import pkg_resources

from kfk.config import KUBECTL_VERSION, STRIMZI_VERSION

version = f"""CLI Version: {pkg_resources.require("strimzi-kafka-cli")[0].version}
Strimzi Version: {STRIMZI_VERSION}
Kubectl Version: {KUBECTL_VERSION}"""


# @click.option('-v', '--version', help='Prints the version', is_flag=True)
@click.version_option(version=1, message=version)
@click.group(name="kfk")
def kfk():
    """Strimzi Kafka CLI."""


if __name__ == "__main__":
    kfk()



import os

import click
import yaml

from kfk import argument_extensions, option_extensions
from kfk.commands.connect import connect
from kfk.commons import (
    add_properties_config_to_resource,
    create_temp_file,
    delete_last_applied_configuration,
    get_properties_from_file,
    get_resource_as_stream,
    raise_exception_for_missing_options,
)
from kfk.config import STRIMZI_PATH, STRIMZI_VERSION
from kfk.constants import SpecialTexts
from kfk.kubectl_command_builder import Kubectl
from kfk.kubernetes_commons import create_using_yaml, delete_using_yaml

CONNECTOR_SKIPPED_PROPERTIES = (
    SpecialTexts.CONNECTOR_NAME,
    SpecialTexts.CONNECTOR_TASKS_MAX,
    SpecialTexts.CONNECTOR_CLASS,
)


@click.option("-n", "--namespace", help="Namespace to use", required=True)
@click.option("-c", "--cluster", help="Connect cluster to use", required=True)
@click.option("--alter", "is_alter", help="Alter the connector.", is_flag=True)
@click.option("--delete", "is_delete", help="Delete the connector.", is_flag=True)
@click.option(
    "-o",
    "--output",
    help=(
        "Output format. One of:"
        " json|yaml|name|go-template|go-template-file|template|templatefile|jsonpath"
        "|jsonpath-file."
    ),
)
@click.option(
    "--describe",
    "is_describe",
    help="List details for the given connector.",
    is_flag=True,
)
@click.argument(
    "config_file",
    type=click.File("r"),
    cls=argument_extensions.NotRequiredIf,
    arguments=["is_describe", "is_delete", "is_list"],
)
@click.option("--create", "is_create", help="Create a new connector.", is_flag=True)
@click.option("--list", "is_list", help="List all available connectors.", is_flag=True)
@click.option(
    "--connector",
    help="Connector Name",
    cls=option_extensions.RequiredIf,
    options=["is_describe", "is_delete"],
)
@connect.command()
def connectors(
    connector,
    is_list,
    is_create,
    config_file,
    is_describe,
    output,
    is_delete,
    is_alter,
    cluster,
    namespace,
):
    """Creates, alters, deletes, describes Kafka Connect connector(s)."""
    if is_list:
        list(cluster, namespace)
    elif is_create:
        create(config_file, cluster, namespace)
    elif is_describe:
        describe(connector, output, namespace)
    elif is_delete:
        delete(connector, cluster, namespace)
    elif is_alter:
        alter(config_file, cluster, namespace)
    else:
        raise_exception_for_missing_options("connectors")


def list(cluster, namespace):
    os.system(
        Kubectl()
        .get()
        .kafkaconnectors()
        .label("strimzi.io/cluster={cluster}")
        .namespace(namespace)
        .build()
        .format(cluster=cluster)
    )


def create(config_file, cluster, namespace):
    with open(
        "{strimzi_path}/examples/connect/source-connector.yaml".format(
            strimzi_path=STRIMZI_PATH
        ).format(version=STRIMZI_VERSION)
    ) as file:
        connector_dict = yaml.full_load(file)

        connector_properties = get_properties_from_file(config_file)

        connector_dict["metadata"]["name"] = connector_properties.get(
            SpecialTexts.CONNECTOR_NAME
        ).data
        connector_dict["metadata"]["labels"]["strimzi.io/cluster"] = cluster

        connector_dict["spec"]["class"] = connector_properties.get(
            SpecialTexts.CONNECTOR_CLASS
        ).data
        connector_dict["spec"]["tasksMax"] = int(
            connector_properties.get(SpecialTexts.CONNECTOR_TASKS_MAX).data
        )
        connector_dict["spec"]["config"] = {}

        add_properties_config_to_resource(
            connector_properties,
            connector_dict["spec"]["config"],
            _return_if_not_skipped,
        )

        connector_yaml = yaml.dump(connector_dict)
        connector_temp_file = create_temp_file(connector_yaml)

        create_using_yaml(connector_temp_file.name, namespace)

        connector_temp_file.close()


def describe(connector, output, namespace):
    if output is not None:
        os.system(
            Kubectl()
            .get()
            .kafkaconnectors(connector)
            .namespace(namespace)
            .output(output)
            .build()
        )
    else:
        os.system(
            Kubectl().describe().kafkaconnectors(connector).namespace(namespace).build()
        )


def delete(connector, cluster, namespace):
    with open(
        "{strimzi_path}/examples/connect/source-connector.yaml".format(
            strimzi_path=STRIMZI_PATH
        ).format(version=STRIMZI_VERSION)
    ) as file:
        connector_dict = yaml.full_load(file)

        connector_dict["metadata"]["name"] = connector
        connector_dict["metadata"]["labels"]["strimzi.io/cluster"] = cluster

        connector_yaml = yaml.dump(connector_dict)
        connector_temp_file = create_temp_file(connector_yaml)

        delete_using_yaml(connector_temp_file.name, namespace)

        connector_temp_file.close()


def alter(config_file, cluster, namespace):
    connector_properties = get_properties_from_file(config_file)

    stream = get_resource_as_stream(
        "kafkaconnectors",
        resource_name=connector_properties.get(SpecialTexts.CONNECTOR_NAME).data,
        namespace=namespace,
    )

    connector_dict = yaml.full_load(stream)

    delete_last_applied_configuration(connector_dict)

    connector_dict["spec"]["class"] = connector_properties.get(
        SpecialTexts.CONNECTOR_CLASS
    ).data
    connector_dict["spec"]["tasksMax"] = int(
        connector_properties.get(SpecialTexts.CONNECTOR_TASKS_MAX).data
    )

    connector_dict["spec"]["config"] = {}

    add_properties_config_to_resource(
        connector_properties, connector_dict["spec"]["config"], _return_if_not_skipped
    )

    connector_yaml = yaml.dump(connector_dict)
    connector_temp_file = create_temp_file(connector_yaml)

    os.system(
        Kubectl()
        .replace()
        .from_file("{topic_temp_file_path}")
        .namespace(namespace)
        .build()
        .format(topic_temp_file_path=connector_temp_file.name)
    )

    connector_temp_file.close()


def _return_if_not_skipped(property_item):
    if property_item[0] not in CONNECTOR_SKIPPED_PROPERTIES:
        return property_item
    else:
        return None



import os

import click
import yaml

from kfk import argument_extensions, option_extensions
from kfk.commands.connect import connect, connectors
from kfk.commons import (
    add_properties_config_to_resource,
    create_temp_file,
    delete_last_applied_configuration,
    get_list_by_split_string,
    get_properties_from_file,
    get_resource_as_stream,
    open_file_in_system_editor,
    raise_exception_for_missing_options,
)
from kfk.config import STRIMZI_PATH, STRIMZI_VERSION
from kfk.constants import (
    COMMA,
    CONNECT_OUTPUT_TYPE_DOCKER,
    EXTENSION_JAR,
    EXTENSION_TAR_GZ,
    EXTENSION_ZIP,
    TRUE,
    SpecialTexts,
)
from kfk.kubectl_command_builder import Kubectl
from kfk.kubernetes_commons import (
    create_registry_secret,
    create_using_yaml,
    delete_object,
    delete_using_yaml,
)
from kfk.messages import Errors, Messages
from kfk.utils import is_valid_url

CONNECT_SKIPPED_PROPERTIES = (
    SpecialTexts.CONNECT_BOOTSTRAP_SERVERS,
    SpecialTexts.CONNECT_IMAGE,
    SpecialTexts.CONNECT_PLUGIN_URL,
    SpecialTexts.CONNECT_PLUGIN_PATH,
)


@click.option("-y", "--yes", "is_yes", help='"Yes" confirmation', is_flag=True)
@click.option("-n", "--namespace", help="Namespace to use")
@click.option("--alter", "is_alter", help="Alters the connect cluster.", is_flag=True)
@click.option(
    "--delete", "is_delete", help="Deletes the connect cluster.", is_flag=True
)
@click.argument("connector_config_files", nargs=-1, type=click.File("r"))
@click.argument(
    "config_file",
    type=click.File("r"),
    cls=argument_extensions.RequiredIf,
    arguments=["is_create"],
)
@click.option(
    "-p", "--registry-password", help="Image registry password for Connect image"
)
@click.option(
    "-u", "--registry-username", help="Image registry username for Connect image"
)
@click.option(
    "--replicas", help="The number of connect replicas for the cluster.", type=int
)
@click.option(
    "--create", "is_create", help="Creates a new connect cluster.", is_flag=True
)
@click.option(
    "--describe",
    "is_describe",
    help="List details for the given cluster.",
    is_flag=True,
)
@click.option(
    "-o",
    "--output",
    help=(
        "Output format. One of:"
        " json|yaml|name|go-template|go-template-file|template|templatefile|jsonpath"
        "|jsonpath-file."
    ),
)
@click.option(
    "--list",
    "is_list",
    help="Lists all available clusters.",
    required=True,
    is_flag=True,
)
@click.option(
    "--cluster",
    help="Connect cluster name",
    required=True,
    cls=option_extensions.NotRequiredIf,
    options=["is_list"],
)
@connect.command()
def clusters(
    cluster,
    is_list,
    is_create,
    replicas,
    registry_username,
    registry_password,
    config_file,
    connector_config_files,
    is_describe,
    is_delete,
    is_alter,
    output,
    namespace,
    is_yes,
):
    """Creates, alters, deletes, describes Kafka Connect cluster(s)"""
    if is_list:
        list(namespace)
    elif is_create:
        create(
            cluster,
            replicas,
            registry_username,
            registry_password,
            config_file,
            connector_config_files,
            namespace,
            is_yes,
        )
    elif is_describe:
        describe(cluster, output, namespace)
    elif is_delete:
        delete(cluster, namespace, is_yes)
    elif is_alter:
        alter(cluster, replicas, config_file, namespace)
    else:
        raise_exception_for_missing_options("connect")


def list(namespace):
    os.system(Kubectl().get().kafkaconnects().namespace(namespace).build())


def create(
    cluster,
    replicas,
    registry_username,
    registry_password,
    config_file,
    connector_config_files,
    namespace,
    is_yes,
):
    with open(
        "{strimzi_path}/examples/connect/kafka-connect.yaml".format(
            strimzi_path=STRIMZI_PATH
        ).format(version=STRIMZI_VERSION)
    ) as file:
        cluster_dict = yaml.full_load(file)

        cluster_dict["metadata"]["name"] = cluster

        cluster_dict["metadata"]["annotations"] = {}
        cluster_dict["metadata"]["annotations"][
            "strimzi.io/use-connector-resources"
        ] = TRUE

        if replicas is not None:
            cluster_dict["spec"]["replicas"] = replicas

        connect_properties = get_properties_from_file(config_file)

        cluster_dict["spec"].pop("tls")

        cluster_dict["spec"]["bootstrapServers"] = connect_properties.get(
            SpecialTexts.CONNECT_BOOTSTRAP_SERVERS
        ).data

        if connect_properties.get(SpecialTexts.CONNECT_PLUGIN_URL) is None:
            cluster_dict["spec"]["image"] = connect_properties.get(
                SpecialTexts.CONNECT_IMAGE
            ).data
        else:
            cluster_dict["spec"]["build"] = {}
            cluster_dict["spec"]["build"]["output"] = {}
            cluster_dict["spec"]["build"]["output"]["type"] = CONNECT_OUTPUT_TYPE_DOCKER
            cluster_dict["spec"]["build"]["output"]["image"] = connect_properties.get(
                SpecialTexts.CONNECT_IMAGE
            ).data
            cluster_dict["spec"]["build"]["output"][
                "pushSecret"
            ] = f"{cluster}-push-secret"
            cluster_dict["spec"]["build"]["plugins"] = []

            for i, plugin_url in enumerate(
                get_list_by_split_string(
                    connect_properties.get(SpecialTexts.CONNECT_PLUGIN_URL).data, COMMA
                ),
                start=1,
            ):
                if not is_valid_url(plugin_url):
                    raise click.ClickException(
                        Errors.NOT_A_VALID_URL + f": {plugin_url}"
                    )

                plugin_dict = {
                    "name": f"connector-{i}",
                    "artifacts": [
                        {"type": _get_plugin_type(plugin_url), "url": plugin_url}
                    ],
                }

                cluster_dict["spec"]["build"]["plugins"].append(plugin_dict)

        cluster_dict["spec"]["config"] = {}

        add_properties_config_to_resource(
            connect_properties, cluster_dict["spec"]["config"], _return_if_not_skipped
        )

        cluster_yaml = yaml.dump(cluster_dict)
        cluster_temp_file = create_temp_file(cluster_yaml)

        if is_yes:
            is_confirmed = True
        else:
            open_file_in_system_editor(cluster_temp_file.name)
            is_confirmed = click.confirm(Messages.CLUSTER_CREATE_CONFIRMATION)

        if is_confirmed:
            if connect_properties.get(SpecialTexts.CONNECT_PLUGIN_URL) is not None:
                username = (
                    registry_username
                    if registry_username is not None
                    else click.prompt(
                        Messages.IMAGE_REGISTRY_USER_NAME, hide_input=False
                    )
                )
                password = (
                    registry_password
                    if registry_password is not None
                    else click.prompt(Messages.IMAGE_REGISTRY_PASSWORD, hide_input=True)
                )

                create_registry_secret(
                    f"{cluster}-push-secret",
                    connect_properties.get(SpecialTexts.CONNECT_IMAGE).data,
                    username,
                    password,
                )

            create_using_yaml(cluster_temp_file.name, namespace)

            for connector_config_file in connector_config_files:
                connectors.create(connector_config_file, cluster, namespace)

        cluster_temp_file.close()


def describe(cluster, output, namespace):
    if output is not None:
        os.system(
            Kubectl()
            .get()
            .kafkaconnects(cluster)
            .namespace(namespace)
            .output(output)
            .build()
        )
    else:
        os.system(
            Kubectl().describe().kafkaconnects(cluster).namespace(namespace).build()
        )


def delete(cluster, namespace, is_yes):
    if is_yes:
        is_confirmed = True
    else:
        is_confirmed = click.confirm(Messages.CLUSTER_DELETE_CONFIRMATION)
    if is_confirmed:
        with open(
            "{strimzi_path}/examples/connect/kafka-connect.yaml".format(
                strimzi_path=STRIMZI_PATH
            ).format(version=STRIMZI_VERSION)
        ) as file:
            cluster_dict = yaml.full_load(file)

            cluster_dict["metadata"]["name"] = cluster

            cluster_yaml = yaml.dump(cluster_dict)
            cluster_temp_file = create_temp_file(cluster_yaml)

            delete_using_yaml(cluster_temp_file.name, namespace)

            cluster_temp_file.close()

        delete_object(f"{cluster}-push-secret", "secret", namespace)


def alter(cluster, replicas, config_file, namespace):
    if config_file is not None or replicas is not None:
        stream = get_resource_as_stream(
            "kafkaconnects", resource_name=cluster, namespace=namespace
        )
        cluster_dict = yaml.full_load(stream)

        delete_last_applied_configuration(cluster_dict)

        if replicas is not None:
            cluster_dict["spec"]["replicas"] = replicas

        if config_file is not None:
            connect_properties = get_properties_from_file(config_file)

            cluster_dict["spec"]["bootstrapServers"] = connect_properties.get(
                SpecialTexts.CONNECT_BOOTSTRAP_SERVERS
            ).data

            if connect_properties.get(SpecialTexts.CONNECT_PLUGIN_URL) is None:
                cluster_dict["spec"].pop("build", None)
                cluster_dict["spec"]["image"] = connect_properties.get(
                    SpecialTexts.CONNECT_IMAGE
                ).data
            else:
                cluster_dict["spec"].pop("image", None)
                cluster_dict["spec"]["build"]["output"][
                    "image"
                ] = connect_properties.get(SpecialTexts.CONNECT_IMAGE).data

                cluster_dict["spec"]["build"]["plugins"] = []

                for i, plugin_url in enumerate(
                    get_list_by_split_string(
                        connect_properties.get(SpecialTexts.CONNECT_PLUGIN_URL).data,
                        COMMA,
                    ),
                    start=1,
                ):
                    if not is_valid_url(plugin_url):
                        raise click.ClickException(
                            Errors.NOT_A_VALID_URL + f": {plugin_url}"
                        )

                    plugin_dict = {
                        "name": f"connector-{i}",
                        "artifacts": [
                            {"type": _get_plugin_type(plugin_url), "url": plugin_url}
                        ],
                    }

                    cluster_dict["spec"]["build"]["plugins"].append(plugin_dict)

            cluster_dict["spec"]["config"] = {}

            add_properties_config_to_resource(
                connect_properties,
                cluster_dict["spec"]["config"],
                _return_if_not_skipped,
            )

        cluster_yaml = yaml.dump(cluster_dict)
        cluster_temp_file = create_temp_file(cluster_yaml)
        os.system(
            Kubectl()
            .replace()
            .from_file("{cluster_temp_file_path}")
            .namespace(namespace)
            .build()
            .format(cluster_temp_file_path=cluster_temp_file.name)
        )
        cluster_temp_file.close()
    else:
        os.system(Kubectl().edit().kafkaconnects(cluster).namespace(namespace).build())


def _return_if_not_skipped(property_item):
    if property_item[0] not in CONNECT_SKIPPED_PROPERTIES:
        return property_item
    else:
        return None


def _get_plugin_type(plugin_url):
    if plugin_url.find(EXTENSION_TAR_GZ) > -1:
        return "tgz"
    elif plugin_url.find(EXTENSION_JAR) > -1:
        return "jar"
    elif plugin_url.find(EXTENSION_ZIP) > -1:
        return "zip"
    else:
        raise click.ClickException(Errors.NO_PLUGIN_TYPE_DETECTED)



class Messages:
    USER_PROVIDED_CONFIG_HEADER = (
        "All user provided configs for brokers in the cluster are:"
    )
    CLUSTER_CREATE_CONFIRMATION = (
        "Are you sure you want to create the cluster with the saved configuration?"
    )
    CLUSTER_DELETE_CONFIRMATION = "Are you sure you want to delete the cluster?"
    IMAGE_REGISTRY_USER_NAME = "Image registry username"
    IMAGE_REGISTRY_PASSWORD = "Image registry password"


class Errors:
    CONFIG_FILE_SHOULD_BE_PROVIDED = (
        "A configuration file should be provided for connect cluster"
    )
    NOT_A_VALID_URL = "Not a valid URL"
    NO_PLUGIN_TYPE_DETECTED = "No plugin type detected"



from unittest import TestCase, mock

from click.testing import CliRunner

from kfk.commands.users import kfk
from kfk.kubectl_command_builder import Kubectl


class TestKfkUsers(TestCase):
    def setUp(self):
        self.runner = CliRunner()
        self.cluster = "my-cluster"
        self.namespace = "kafka"
        self.user = "my-user"

    def test_no_option(self):
        result = self.runner.invoke(
            kfk,
            ["users", "--user", self.user, "-c", self.cluster, "-n", self.namespace],
        )
        assert result.exit_code == 1
        assert "Missing options: kfk users" in result.output

    @mock.patch("kfk.commands.users.os")
    def test_list_users(self, mock_os):
        result = self.runner.invoke(
            kfk, ["users", "--list", "-c", self.cluster, "-n", self.namespace]
        )
        assert result.exit_code == 0
        mock_os.system.assert_called_with(
            Kubectl()
            .get()
            .kafkausers()
            .label("strimzi.io/cluster={cluster}")
            .namespace(self.namespace)
            .build()
            .format(cluster=self.cluster)
        )

    @mock.patch("kfk.commands.users.os")
    def test_describe_user(self, mock_os):
        result = self.runner.invoke(
            kfk,
            [
                "users",
                "--describe",
                "--user",
                self.user,
                "-c",
                self.cluster,
                "-n",
                self.namespace,
            ],
        )
        assert result.exit_code == 0
        mock_os.system.assert_called_with(
            Kubectl().describe().kafkausers(self.user).namespace(self.namespace).build()
        )

    @mock.patch("kfk.commands.users.os")
    def test_describe_user_output_yaml(self, mock_os):
        result = self.runner.invoke(
            kfk,
            [
                "users",
                "--describe",
                "--user",
                self.user,
                "-c",
                self.cluster,
                "-n",
                self.namespace,
                "-o",
                "yaml",
            ],
        )
        assert result.exit_code == 0
        mock_os.system.assert_called_with(
            Kubectl()
            .get()
            .kafkausers(self.user)
            .namespace(self.namespace)
            .output("yaml")
            .build()
        )

    @mock.patch("kfk.commands.users.create_temp_file")
    @mock.patch("kfk.commands.users.create_using_yaml")
    def test_create_user(self, mock_create_using_yaml, mock_create_temp_file):
        result = self.runner.invoke(
            kfk,
            [
                "users",
                "--create",
                "--user",
                self.user,
                "--authentication-type",
                "tls",
                "-c",
                self.cluster,
                "-n",
                self.namespace,
            ],
        )

        assert result.exit_code == 0

        with open("tests/files/yaml/user_with_authentication_tls.yaml") as file:
            expected_user_yaml = file.read()
            result_user_yaml = mock_create_temp_file.call_args[0][0]
            assert expected_user_yaml == result_user_yaml

        mock_create_using_yaml.assert_called_once()

    def test_create_user_without_auth(self):
        result = self.runner.invoke(
            kfk,
            [
                "users",
                "--create",
                "--user",
                self.user,
                "-c",
                self.cluster,
                "-n",
                self.namespace,
            ],
        )
        assert result.exit_code == 2

    def test_create_user_with_wrong_authentication_type(self):
        result = self.runner.invoke(
            kfk,
            [
                "users",
                "--create",
                "--user",
                self.user,
                "--authentication-type",
                "auth",
                "--authorization-type",
                "simple",
                "-c",
                self.cluster,
                "-n",
                self.namespace,
            ],
        )
        assert result.exit_code == 2

    def test_create_user_with_wrong_authorization_type(self):
        result = self.runner.invoke(
            kfk,
            [
                "users",
                "--create",
                "--user",
                self.user,
                "--authentication-type",
                "tls",
                "--authorization-type",
                "auth",
                "-c",
                self.cluster,
                "-n",
                self.namespace,
            ],
        )
        assert result.exit_code == 2

    @mock.patch("kfk.commands.users.delete_using_yaml")
    def test_delete_user(self, mock_delete_using_yaml):
        result = self.runner.invoke(
            kfk,
            [
                "users",
                "--delete",
                "--user",
                self.user,
                "-c",
                self.cluster,
                "-n",
                self.namespace,
            ],
        )
        assert result.exit_code == 0

        mock_delete_using_yaml.assert_called_once()

    @mock.patch("kfk.commands.users.create_temp_file")
    @mock.patch("kfk.commons.get_resource_yaml")
    @mock.patch("kfk.commands.users.os")
    def test_alter_user_with_no_params(
        self, mock_os, mock_get_resource_yaml, mock_create_temp_file
    ):
        with open("tests/files/yaml/user_with_authentication_tls.yaml") as file:
            expected_user_yaml = file.read()
            mock_get_resource_yaml.return_value = expected_user_yaml

            result = self.runner.invoke(
                kfk,
                [
                    "users",
                    "--alter",
                    "--user",
                    self.user,
                    "-c",
                    self.cluster,
                    "-n",
                    self.namespace,
                ],
            )
            assert result.exit_code == 0

            result_user_yaml = mock_create_temp_file.call_args[0][0]
            assert expected_user_yaml == result_user_yaml

    @mock.patch("kfk.commands.users.create_temp_file")
    @mock.patch("kfk.commons.get_resource_yaml")
    @mock.patch("kfk.commands.users.os")
    def test_alter_user_for_authentication(
        self, mock_os, mock_get_resource_yaml, mock_create_temp_file
    ):
        with open("tests/files/yaml/user_with_authentication_tls.yaml") as file:
            user_yaml = file.read()
            mock_get_resource_yaml.return_value = user_yaml

            result = self.runner.invoke(
                kfk,
                [
                    "users",
                    "--alter",
                    "--user",
                    self.user,
                    "--authentication-type",
                    "scram-sha-512",
                    "-c",
                    self.cluster,
                    "-n",
                    self.namespace,
                ],
            )
            assert result.exit_code == 0

            with open("tests/files/yaml/user_with_authentication_scram.yaml") as file:
                expected_user_yaml = file.read()
                result_user_yaml = mock_create_temp_file.call_args[0][0]
                assert expected_user_yaml == result_user_yaml

    @mock.patch("kfk.commands.users.create_temp_file")
    @mock.patch("kfk.commons.get_resource_yaml")
    @mock.patch("kfk.commands.users.os")
    def test_alter_user_for_authorization(
        self, mock_os, mock_get_resource_yaml, mock_create_temp_file
    ):
        with open("tests/files/yaml/user_with_authentication_tls.yaml") as file:
            user_yaml = file.read()
            mock_get_resource_yaml.return_value = user_yaml

            result = self.runner.invoke(
                kfk,
                [
                    "users",
                    "--alter",
                    "--user",
                    self.user,
                    "--authorization-type",
                    "simple",
                    "--add-acl",
                    "--operation",
                    "Read",
                    "--resource-type",
                    "topic",
                    "--resource-name",
                    "my-topic",
                    "-c",
                    self.cluster,
                    "-n",
                    self.namespace,
                ],
            )
            assert result.exit_code == 0

            with open(
                "tests/files/yaml/user_with_authorization_with_one_topic_acl.yaml"
            ) as file:
                expected_user_yaml = file.read()
                result_user_yaml = mock_create_temp_file.call_args[0][0]
                assert expected_user_yaml == result_user_yaml

    @mock.patch("kfk.commands.users.create_temp_file")
    @mock.patch("kfk.commons.get_resource_yaml")
    @mock.patch("kfk.commands.users.os")
    def test_alter_user_for_authorization_none(
        self, mock_os, mock_get_resource_yaml, mock_create_temp_file
    ):
        with open("tests/files/yaml/user_with_authentication_tls.yaml") as file:
            user_yaml = file.read()
            mock_get_resource_yaml.return_value = user_yaml

            result = self.runner.invoke(
                kfk,
                [
                    "users",
                    "--alter",
                    "--user",
                    self.user,
                    "--authorization-type",
                    "none",
                    "--add-acl",
                    "--operation",
                    "Read",
                    "--resource-type",
                    "topic",
                    "--resource-name",
                    "my-topic",
                    "-c",
                    self.cluster,
                    "-n",
                    self.namespace,
                ],
            )
            assert result.exit_code == 0

            with open("tests/files/yaml/user_with_authentication_tls.yaml") as file:
                expected_user_yaml = file.read()
                result_user_yaml = mock_create_temp_file.call_args[0][0]
                assert expected_user_yaml == result_user_yaml

    @mock.patch("kfk.commands.users.create_temp_file")
    @mock.patch("kfk.commons.get_resource_yaml")
    @mock.patch("kfk.commands.users.os")
    def test_alter_user_for_authorization_with_two_acl_operation(
        self, mock_os, mock_get_resource_yaml, mock_create_temp_file
    ):
        with open("tests/files/yaml/user_with_authentication_tls.yaml") as file:
            user_yaml = file.read()
            mock_get_resource_yaml.return_value = user_yaml

            result = self.runner.invoke(
                kfk,
                [
                    "users",
                    "--alter",
                    "--user",
                    self.user,
                    "--authorization-type",
                    "simple",
                    "--add-acl",
                    "--operation",
                    "Read",
                    "--operation",
                    "Write",
                    "--resource-type",
                    "topic",
                    "--resource-name",
                    "my-topic",
                    "-c",
                    self.cluster,
                    "-n",
                    self.namespace,
                ],
            )
            assert result.exit_code == 0

            with open(
                "tests/files/yaml/user_with_authorization_with_two_topic_acls.yaml"
            ) as file:
                expected_user_yaml = file.read()
                result_user_yaml = mock_create_temp_file.call_args[0][0]

                assert expected_user_yaml == result_user_yaml

    @mock.patch("kfk.commands.users.create_temp_file")
    @mock.patch("kfk.commons.get_resource_yaml")
    @mock.patch("kfk.commands.users.os")
    def test_alter_user_for_authorization_none_with_two_acl_operation(
        self, mock_os, mock_get_resource_yaml, mock_create_temp_file
    ):
        with open(
            "tests/files/yaml/user_with_authorization_with_two_topic_acls.yaml"
        ) as file:
            user_yaml = file.read()
            mock_get_resource_yaml.return_value = user_yaml

            result = self.runner.invoke(
                kfk,
                [
                    "users",
                    "--alter",
                    "--user",
                    self.user,
                    "--authorization-type",
                    "none",
                    "--add-acl",
                    "--operation",
                    "Read",
                    "--operation",
                    "Write",
                    "--resource-type",
                    "topic",
                    "--resource-name",
                    "my-topic",
                    "-c",
                    self.cluster,
                    "-n",
                    self.namespace,
                ],
            )
            assert result.exit_code == 0

            with open("tests/files/yaml/user_with_authentication_tls.yaml") as file:
                expected_user_yaml = file.read()
                result_user_yaml = mock_create_temp_file.call_args[0][0]

                assert expected_user_yaml == result_user_yaml

    @mock.patch("kfk.commands.users.create_temp_file")
    @mock.patch("kfk.commons.get_resource_yaml")
    @mock.patch("kfk.commands.users.os")
    def test_alter_user_for_authorization_delete_acls_for_one_operation(
        self, mock_os, mock_get_resource_yaml, mock_create_temp_file
    ):
        with open(
            "tests/files/yaml/user_with_authorization_with_three_topic_acls.yaml"
        ) as file:
            user_yaml = file.read()
            mock_get_resource_yaml.return_value = user_yaml

            result = self.runner.invoke(
                kfk,
                [
                    "users",
                    "--alter",
                    "--user",
                    self.user,
                    "--authorization-type",
                    "simple",
                    "--delete-acl",
                    "--operation",
                    "Describe",
                    "--resource-type",
                    "topic",
                    "--resource-name",
                    "my-topic",
                    "-c",
                    self.cluster,
                    "-n",
                    self.namespace,
                ],
            )
            assert result.exit_code == 0

            with open(
                "tests/files/yaml/user_with_authorization_with_two_topic_acls.yaml"
            ) as file:
                expected_user_yaml = file.read()
                result_user_yaml = mock_create_temp_file.call_args[0][0]

                assert expected_user_yaml == result_user_yaml

    @mock.patch("kfk.commands.users.create_temp_file")
    @mock.patch("kfk.commons.get_resource_yaml")
    @mock.patch("kfk.commands.users.os")
    def test_alter_user_for_authorization_delete_acls_for_two_operation(
        self, mock_os, mock_get_resource_yaml, mock_create_temp_file
    ):
        with open(
            "tests/files/yaml/user_with_authorization_with_three_topic_acls.yaml"
        ) as file:
            user_yaml = file.read()
            mock_get_resource_yaml.return_value = user_yaml

            result = self.runner.invoke(
                kfk,
                [
                    "users",
                    "--alter",
                    "--user",
                    self.user,
                    "--authorization-type",
                    "simple",
                    "--delete-acl",
                    "--operation",
                    "Describe",
                    "--operation",
                    "Write",
                    "--resource-type",
                    "topic",
                    "--resource-name",
                    "my-topic",
                    "-c",
                    self.cluster,
                    "-n",
                    self.namespace,
                ],
            )
            assert result.exit_code == 0

            with open(
                "tests/files/yaml/user_with_authorization_with_one_topic_acl.yaml"
            ) as file:
                expected_user_yaml = file.read()
                result_user_yaml = mock_create_temp_file.call_args[0][0]

                assert expected_user_yaml == result_user_yaml

    @mock.patch("kfk.commands.users.create_temp_file")
    @mock.patch("kfk.commons.get_resource_yaml")
    @mock.patch("kfk.commands.users.os")
    def test_alter_user_for_quota(
        self, mock_os, mock_get_resource_yaml, mock_create_temp_file
    ):
        with open("tests/files/yaml/user_with_authentication_tls.yaml") as file:
            user_yaml = file.read()
            mock_get_resource_yaml.return_value = user_yaml

            result = self.runner.invoke(
                kfk,
                [
                    "users",
                    "--alter",
                    "--user",
                    self.user,
                    "--authentication-type",
                    "scram-sha-512",
                    "--quota",
                    "request_percentage=55",
                    "-c",
                    self.cluster,
                    "-n",
                    self.namespace,
                ],
            )
            assert result.exit_code == 0

            with open("tests/files/yaml/user_with_one_quota.yaml") as file:
                expected_user_yaml = file.read()
                result_user_yaml = mock_create_temp_file.call_args[0][0]
                assert expected_user_yaml == result_user_yaml

    @mock.patch("kfk.commands.users.create_temp_file")
    @mock.patch("kfk.commons.get_resource_yaml")
    @mock.patch("kfk.commands.users.os")
    def test_alter_user_for_two_quotas(
        self, mock_os, mock_get_resource_yaml, mock_create_temp_file
    ):
        with open("tests/files/yaml/user_with_authentication_tls.yaml") as file:
            user_yaml = file.read()
            mock_get_resource_yaml.return_value = user_yaml

            result = self.runner.invoke(
                kfk,
                [
                    "users",
                    "--alter",
                    "--user",
                    self.user,
                    "--authentication-type",
                    "scram-sha-512",
                    "--quota",
                    "request_percentage=55",
                    "--quota",
                    "consumer_byte_rate=2097152",
                    "-c",
                    self.cluster,
                    "-n",
                    self.namespace,
                ],
            )
            assert result.exit_code == 0

            with open("tests/files/yaml/user_with_two_quotas.yaml") as file:
                expected_user_yaml = file.read()
                result_user_yaml = mock_create_temp_file.call_args[0][0]
                assert expected_user_yaml == result_user_yaml

    @mock.patch("kfk.commands.users.create_temp_file")
    @mock.patch("kfk.commons.get_resource_yaml")
    @mock.patch("kfk.commands.users.os")
    def test_alter_user_with_two_quotas_delete_one_quota(
        self, mock_os, mock_get_resource_yaml, mock_create_temp_file
    ):
        with open("tests/files/yaml/user_with_two_quotas.yaml") as file:
            user_yaml = file.read()
            mock_get_resource_yaml.return_value = user_yaml

            result = self.runner.invoke(
                kfk,
                [
                    "users",
                    "--alter",
                    "--user",
                    self.user,
                    "--delete-quota",
                    "consumer_byte_rate",
                    "-c",
                    self.cluster,
                    "-n",
                    self.namespace,
                ],
            )
            assert result.exit_code == 0

            with open("tests/files/yaml/user_with_one_quota.yaml") as file:
                expected_user_yaml = file.read()
                result_user_yaml = mock_create_temp_file.call_args[0][0]
                assert expected_user_yaml == result_user_yaml

    @mock.patch("kfk.commands.users.create_temp_file")
    @mock.patch("kfk.commons.get_resource_yaml")
    @mock.patch("kfk.commands.users.os")
    def test_alter_user_with_two_quotas_delete_two_quotas(
        self, mock_os, mock_get_resource_yaml, mock_create_temp_file
    ):
        with open("tests/files/yaml/user_with_two_quotas.yaml") as file:
            user_yaml = file.read()
            mock_get_resource_yaml.return_value = user_yaml

            result = self.runner.invoke(
                kfk,
                [
                    "users",
                    "--alter",
                    "--user",
                    self.user,
                    "--delete-quota",
                    "consumerByteRate",
                    "--delete-quota",
                    "requestPercentage",
                    "-c",
                    self.cluster,
                    "-n",
                    self.namespace,
                ],
            )
            assert result.exit_code == 0

            with open("tests/files/yaml/user_with_quotas_empty.yaml") as file:
                expected_user_yaml = file.read()
                result_user_yaml = mock_create_temp_file.call_args[0][0]
                assert expected_user_yaml == result_user_yaml



import os

import click

from kfk.commands import users
from kfk.commands.main import kfk
from kfk.commons import raise_exception_for_missing_options
from kfk.constants import COLON
from kfk.kubectl_command_builder import Kubectl
from kfk.option_extensions import NotRequiredIf


@click.option("-n", "--namespace", help="Namespace to use.", required=True)
@click.option("-c", "--kafka-cluster", help="Cluster to use.", required=True)
@click.option("--remove", help="Indicates you are trying to remove ACLs.", is_flag=True)
@click.option(
    "--resource-pattern-type",
    help=(
        "The type of the resource pattern or <ANY|MATCH|LITERAL|PREFIXED> pattern"
        " filter. When adding acls, this should be a specific pattern type, e.g."
        " 'literal' or 'prefixed'. (default: literal)"
    ),
    default="literal",
)
@click.option(
    "--deny-host",
    help="Host which User will not have access. (default: *)",
    default="*",
)
@click.option(
    "--allow-host", help="Host which User will have access. (default: *)", default="*"
)
@click.option(
    "--operation",
    "operation_tuple",
    help="Operation that is being allowed or denied. (default: All)",
    default=["All"],
    multiple=True,
)
@click.option(
    "--deny-principal",
    help=(
        "principal is in principalType:name format. By default anyone not added through"
        " --allow-principal is denied access. You only need to use this option as"
        " negation to already allowed set."
    ),
    required=True,
    cls=NotRequiredIf,
    options=["list", "allow_principal"],
)
@click.option(
    "--allow-principal",
    help=(
        "principal is in principalType:name principal format. Note that principalType"
        " must be supported by the Authorizer being used."
    ),
    required=True,
    cls=NotRequiredIf,
    options=["list", "deny_principal"],
)
@click.option("--add", help="Indicates you are trying to add ACLs.", is_flag=True)
@click.option("--group", help="Consumer Group ACLs.")
@click.option("--cluster", help="Cluster ACLs.")
@click.option("--topic", help="Topic ACLs.")
@click.option(
    "--list",
    help=(
        "List ACLs for the specified resource, use --topic <topic> or --group <group>"
        " or --cluster to specify a resource."
    ),
    is_flag=True,
)
@kfk.command()
def acls(
    list,
    topic,
    cluster,
    group,
    add,
    allow_principal,
    deny_principal,
    operation_tuple,
    allow_host,
    deny_host,
    resource_pattern_type,
    remove,
    kafka_cluster,
    namespace,
):
    """Manages ACLs on Kafka."""
    if list:
        native_command = (
            "bin/kafka-acls.sh --authorizer-properties"
            " zookeeper.connect=localhost:12181 --list {topic}{cluster} {group}"
        )
        os.system(
            Kubectl()
            .exec("-it", "{kafka_cluster}-zookeeper-0")
            .container("zookeeper")
            .namespace(namespace)
            .exec_command(native_command)
            .build()
            .format(
                kafka_cluster=kafka_cluster,
                topic=(topic and "--topic " + topic or ""),
                cluster=(cluster and "--cluster " + cluster or ""),
                group=(group and "--group " + group or ""),
            )
        )
    elif add or remove:
        add_or_remove(
            topic,
            cluster,
            group,
            add,
            remove,
            allow_principal,
            deny_principal,
            operation_tuple,
            allow_host,
            deny_host,
            resource_pattern_type,
            kafka_cluster,
            namespace,
        )
    else:
        raise_exception_for_missing_options("acls")


def add_or_remove(
    topic,
    cluster,
    group,
    add,
    remove,
    allow_principal,
    deny_principal,
    operation_tuple,
    allow_host,
    deny_host,
    resource_pattern_type,
    kafka_cluster,
    namespace,
):
    resource_type_dict = _get_resource_type_dict(topic, cluster, group)

    if allow_principal:
        type = "allow"
        # TODO: click exception here
        allow_principal_arr = allow_principal.split(COLON)
        principal_type = allow_principal_arr[0]
        principal_name = allow_principal_arr[1]
        host = allow_host
    else:
        type = "deny"
        # TODO: click exception here
        deny_principal_arr = deny_principal.split(COLON)
        principal_type = deny_principal_arr[0]
        principal_name = deny_principal_arr[1]
        host = deny_host

    if principal_type == "User":
        for resource_type, resource_name in resource_type_dict.items():
            users.alter(
                principal_name,
                None,
                "simple",
                add,
                remove,
                operation_tuple,
                host,
                type,
                resource_type,
                resource_name,
                resource_pattern_type,
                tuple(),
                tuple(),
                kafka_cluster,
                namespace,
            )


def _get_resource_type_dict(topic, cluster, group):
    resource_type_dict = {}

    if topic is not None:
        resource_type_dict["topic"] = topic
    if cluster is not None:
        resource_type_dict["cluster"] = cluster
    if group is not None:
        resource_type_dict["group"] = group

    return resource_type_dict



from kfk.config import KUBECTL_PATH
from kfk.constants import SPACE


class Kubectl:
    def __init__(self):
        self.cmd_str = KUBECTL_PATH

    def version(self, *vals):
        self.cmd_str = self.cmd_str + SPACE + "version"
        for val in vals:
            self.cmd_str = self.cmd_str + SPACE + val
        return self

    def get(self):
        self.cmd_str = self.cmd_str + SPACE + "get"
        return self

    def apply(self):
        self.cmd_str = self.cmd_str + SPACE + "apply"
        return self

    def replace(self):
        self.cmd_str = self.cmd_str + SPACE + "replace"
        return self

    def describe(self):
        self.cmd_str = self.cmd_str + SPACE + "describe"
        return self

    def edit(self):
        self.cmd_str = self.cmd_str + SPACE + "edit"
        return self

    def exec(self, flag, pod_name):
        self.cmd_str = self.cmd_str + SPACE + "exec" + SPACE + flag + SPACE + pod_name
        return self

    def exec_command(self, command):
        self.cmd_str = (
            self.cmd_str
            + SPACE
            + "--"
            + SPACE
            + "bash -c"
            + SPACE
            + '"'
            + command
            + '"'
        )
        return self

    def cp(self, source_path, destination_path):
        self.cmd_str = (
            self.cmd_str + SPACE + "cp" + SPACE + source_path + SPACE + destination_path
        )
        return self

    def kafkas(self, *vals):
        return self.resource("kafkas", *vals)

    def kafkaconnects(self, *vals):
        return self.resource("kafkaconnects", *vals)

    def kafkaconnectors(self, *vals):
        return self.resource("kafkaconnectors", *vals)

    def kafkatopics(self, *vals):
        return self.resource("kafkatopics", *vals)

    def kafkausers(self, *vals):
        return self.resource("kafkausers", *vals)

    def configmap(self, *vals):
        return self.resource("configmap", *vals)

    def secret(self, *vals):
        return self.resource("secret", *vals)

    def resource(self, resource_name, *vals):
        self.cmd_str = self.cmd_str + SPACE + resource_name
        for val in vals:
            self.cmd_str = self.cmd_str + SPACE + val
        return self

    def label(self, val):
        self.cmd_str = self.cmd_str + SPACE + "-l" + SPACE + val
        return self

    def namespace(self, *vals):
        if len(vals) > 0 and vals[0]:
            self.cmd_str = self.cmd_str + SPACE + "-n" + SPACE + vals[0]
        else:
            self.cmd_str = self.cmd_str + SPACE + "--all-namespaces"
        return self

    def container(self, val):
        self.cmd_str = self.cmd_str + SPACE + "-c" + SPACE + val
        return self

    def output(self, val):
        self.cmd_str = self.cmd_str + SPACE + "-o" + SPACE + val
        return self

    def from_file(self, val):
        self.cmd_str = self.cmd_str + SPACE + "-f" + SPACE + val
        return self

    def build(self):
        return self.cmd_str



from kfk.commands.acls import acls
from kfk.commands.clusters import clusters
from kfk.commands.configs import configs
from kfk.commands.connect.clusters import clusters
from kfk.commands.connect.connectors import connectors
from kfk.commands.console import console_consumer, console_producer
from kfk.commands.env import env
from kfk.commands.main import kfk
from kfk.commands.operator import operator
from kfk.commands.topics import topics
from kfk.commands.users import users
from kfk.setup import setup

setup()



import base64
import json
import re
import sys
from os import path

import yaml
from kubernetes import client, config

config.load_kube_config()
api_client = client.ApiClient()


def yaml_object_argument_filter(func):
    def inner(k8s_api, yml_object, kind, **kwargs):
        if kind != "custom_object":
            kwargs.pop("version")
            kwargs.pop("group")
            kwargs.pop("plural")
        return func(k8s_api, yml_object, kind, **kwargs)

    return inner


def delete_object(name, resource_type, namespace):
    k8s_api = client.CoreV1Api(api_client)
    _delete_object(k8s_api, name, resource_type, namespace=namespace)
    print(f"{resource_type.capitalize()} `{name}` deleted.")


def create_registry_secret(
    name: str,
    registry: str,
    username: str,
    password: str,
):
    core_api = client.CoreV1Api(api_client)

    auth = base64.b64encode(f"{username}:{password}".encode("utf-8")).decode("utf-8")

    docker_config_dict = {
        "auths": {
            registry: {
                "username": username,
                "password": password,
                "email": "",
                "auth": auth,
            }
        }
    }

    docker_config = base64.b64encode(
        json.dumps(docker_config_dict).encode("utf-8")
    ).decode("utf-8")

    core_api.create_namespaced_secret(
        namespace="default",
        body=client.V1Secret(
            metadata=client.V1ObjectMeta(
                name=name,
            ),
            type="kubernetes.io/dockerconfigjson",
            data={".dockerconfigjson": docker_config},
        ),
    )

    print(f"Registry Secret `{name}` created.")


def create_using_yaml(file_path, namespace):
    _operate_using_yaml(
        api_client,
        file_path,
        "create",
        yaml_objects=None,
        verbose=True,
        namespace=namespace,
    )


def delete_using_yaml(file_path, namespace):
    _operate_using_yaml(
        api_client,
        file_path,
        "delete",
        yaml_objects=None,
        verbose=True,
        namespace=namespace,
    )


def _operate_using_yaml(
    k8s_client,
    yaml_file=None,
    operation=None,
    yaml_objects=None,
    verbose=False,
    namespace="default",
    **kwargs,
):
    def _operate_with(objects):
        failures = []
        k8s_objects = []
        for yml_object in objects:
            if yml_object is None:
                continue
            try:
                created = _operate_using_dict(
                    k8s_client,
                    yml_object,
                    operation,
                    verbose,
                    namespace=namespace,
                    **kwargs,
                )
                k8s_objects.append(created)
            except FailToExecuteError as failure:
                failures.extend(failure.api_exceptions)
        if failures:
            raise FailToExecuteError(failures)
        return k8s_objects

    if yaml_objects:
        yml_object_all = yaml_objects
        return _operate_with(yml_object_all)
    elif yaml_file:
        with open(path.abspath(yaml_file)) as f:
            yml_object_all = yaml.safe_load_all(f)
            return _operate_with(yml_object_all)
    else:
        raise ValueError(
            "One of `yaml_file` or `yaml_objects` arguments must be provided"
        )


def _operate_using_dict(
    k8s_client, yml_object, operation, verbose, namespace="default", **kwargs
):
    api_exceptions = []
    if "List" in yml_object["kind"]:
        kind = yml_object["kind"].replace("List", "")
        for yml_doc in yml_object["items"]:
            if kind != "":
                yml_doc["apiVersion"] = yml_object["apiVersion"]
                yml_doc["kind"] = kind
            try:
                _operate_using_dict_single_object(
                    k8s_client,
                    yml_doc,
                    operation,
                    verbose,
                    namespace=namespace,
                    **kwargs,
                )
            except client.rest.ApiException as api_exception:
                api_exceptions.append(api_exception)
    else:
        try:
            _operate_using_dict_single_object(
                k8s_client,
                yml_object,
                operation,
                verbose,
                namespace=namespace,
                **kwargs,
            )
        except client.rest.ApiException as api_exception:
            api_exceptions.append(api_exception)

    if api_exceptions:
        raise FailToExecuteError(api_exceptions)


def _operate_using_dict_single_object(
    k8s_client, yml_object, operation, verbose=False, namespace="default", **kwargs
):
    object_type = ""
    # get group and version from apiVersion
    group, _, version = yml_object["apiVersion"].partition("/")
    if version == "":
        version = group
        group = "core"
    # Take care for the case e.g. api_type is "apiextensions.k8s.io"
    group_prefix = "".join(group.rsplit(".k8s.io", 1))
    # convert group name from DNS subdomain format to
    # python class name convention
    group_prefix = "".join(word.capitalize() for word in group_prefix.split("."))
    func = "{0}{1}Api".format(group_prefix, version.capitalize())

    try:
        k8s_api = getattr(client, func)(k8s_client)
    except AttributeError:
        func = "CustomObjectsApi"
        k8s_api = getattr(client, func)(k8s_client)
        object_type = "custom_object"

    kind = yml_object["kind"]
    kind_snake_case = re.sub("(.)([A-Z][a-z]+)", r"\1_\2", kind)
    object_type_from_kind = re.sub(
        "([a-z0-9])([A-Z])", r"\1_\2", kind_snake_case
    ).lower()
    name = yml_object["metadata"]["name"]

    if not object_type:
        object_type = object_type_from_kind

    getattr(sys.modules[__name__], f"_{operation}_using_yaml_object")(
        k8s_api,
        yml_object,
        object_type,
        version=version,
        group=group,
        plural=kind.lower() + "s",
        namespace=namespace,
    )
    if verbose:
        msg = f"{kind} `{name}` {operation}d."
        print(msg)


@yaml_object_argument_filter
def _create_using_yaml_object(k8s_api, yml_object, object_type, **kwargs):
    if hasattr(k8s_api, f"create_namespaced_{object_type}"):
        if "namespace" in yml_object["metadata"]:
            namespace = yml_object["metadata"]["namespace"]
            kwargs["namespace"] = namespace
        resp = getattr(k8s_api, f"create_namespaced_{object_type}")(
            body=yml_object, **kwargs
        )
    else:
        kwargs.pop("namespace", None)
        resp = getattr(k8s_api, f"create_{object_type}")(body=yml_object, **kwargs)
    return resp


@yaml_object_argument_filter
def _delete_using_yaml_object(k8s_api, yml_object, object_type, **kwargs):
    if "namespace" in yml_object["metadata"]:
        namespace = yml_object["metadata"]["namespace"]
        kwargs["namespace"] = namespace
    name = yml_object["metadata"]["name"]
    _delete_object(k8s_api, name, object_type, **kwargs)


def _delete_object(k8s_api, name, object_type, delete_options_version="V1", **kwargs):
    try:
        if hasattr(k8s_api, f"delete_namespaced_{object_type}"):
            resp = getattr(k8s_api, f"delete_namespaced_{object_type}")(
                name=name,
                body=getattr(client, f"{delete_options_version}DeleteOptions")(
                    propagation_policy="Background", grace_period_seconds=5
                ),
                **kwargs,
            )
        else:
            kwargs.pop("namespace", None)
            resp = getattr(k8s_api, f"delete_{object_type}")(
                name=name,
                body=getattr(client, f"{delete_options_version}DeleteOptions")(
                    propagation_policy="Background", grace_period_seconds=5
                ),
                **kwargs,
            )
        return resp
    except client.rest.ApiException as api_exception:
        if api_exception.reason != "Not Found":
            raise api_exception


class FailToExecuteError(Exception):
    """An exception class for handling error if an error occurred when handling
    a yaml file during creation or deletion of the resource."""

    def __init__(self, api_exceptions):
        self.api_exceptions = api_exceptions

    def __str__(self):
        msg = ""
        for api_exception in self.api_exceptions:
            msg += "Error from server ({0}):{1}".format(
                api_exception.reason, api_exception.body
            )
        return msg



from unittest import TestCase, mock

from click.testing import CliRunner

from kfk.commands.operator import kfk


class TestKfkOperator(TestCase):
    def setUp(self):
        self.runner = CliRunner()
        self.cluster = "my-cluster"
        self.namespace = "kafka"
        self.installation_file_count = 28

    @mock.patch("kfk.commands.operator.create_using_yaml")
    def test_install_strimzi(self, mock_create_using_yaml):
        result = self.runner.invoke(
            kfk, ["operator", "--install", "-n", self.namespace]
        )
        assert result.exit_code == 0
        assert mock_create_using_yaml.call_count == self.installation_file_count

    @mock.patch("kfk.commands.operator.delete_using_yaml")
    def test_uninstall_strimzi(self, mock_delete_using_yaml):
        result = self.runner.invoke(
            kfk, ["operator", "--uninstall", "-n", self.namespace]
        )
        assert result.exit_code == 0
        assert mock_delete_using_yaml.call_count == self.installation_file_count



import os

import click

from kfk.commands.main import kfk
from kfk.commons import create_temp_file, raise_exception_for_missing_options
from kfk.config import STRIMZI_PATH
from kfk.constants import SpecialTexts
from kfk.kubernetes_commons import create_using_yaml, delete_using_yaml


@click.option("-n", "--namespace", help="Namespace to use", required=True)
@click.option(
    "--uninstall",
    "is_uninstall",
    help="Uninstalls Strimzi Kafka Operator",
    is_flag=True,
)
@click.option(
    "--install", "is_install", help="Installs Strimzi Kafka Operator", is_flag=True
)
@kfk.command()
def operator(is_install, is_uninstall, namespace):
    """Installs/Uninstalls Strimzi Kafka Operator."""

    for directory_name, dirs, files in os.walk(
        "{strimzi_path}/install/cluster-operator/".format(strimzi_path=STRIMZI_PATH)
    ):
        for file_name in files:
            file_path = os.path.join(directory_name, file_name)

            if SpecialTexts.OPERATOR_ROLE_BINDING in file_name:
                with open(file_path) as file:
                    stream = file.read().replace(
                        SpecialTexts.OPERATOR_MY_PROJECT, namespace
                    )
                    temp_file = create_temp_file(stream)
                    file_path = temp_file.name
            if is_install:
                create_using_yaml(file_path, namespace)
            elif is_uninstall:
                delete_using_yaml(file_path, namespace)
            else:
                raise_exception_for_missing_options("operator")
                break



from unittest import TestCase, mock

from click.testing import CliRunner

from kfk.commands.console import kfk
from kfk.kubectl_command_builder import Kubectl


class TestKfkConsole(TestCase):
    def setUp(self):
        self.runner = CliRunner()
        self.cluster = "my-cluster"
        self.namespace = "kafka"
        self.topic = "my-topic"

    @mock.patch("kfk.commands.console.os")
    def test_console_consumer(self, mock_os):
        result = self.runner.invoke(
            kfk,
            [
                "console-consumer",
                "--topic",
                self.topic,
                "-c",
                self.cluster,
                "-n",
                self.namespace,
            ],
        )
        assert result.exit_code == 0

        native_command = (
            "bin/kafka-console-consumer.sh --bootstrap-server"
            " my-cluster-kafka-brokers:9092 --topic {topic} "
        )
        mock_os.system.assert_called_with(
            Kubectl()
            .exec("-it", "{cluster}-kafka-0")
            .container("kafka")
            .namespace(self.namespace)
            .exec_command(native_command)
            .build()
            .format(cluster=self.cluster, topic=self.topic)
        )

    @mock.patch("kfk.commons.transfer_file_to_container")
    @mock.patch("kfk.commands.console.os")
    def test_console_consumer_with_consumer_config(
        self, mock_os, mock_transfer_file_to_container
    ):
        result = self.runner.invoke(
            kfk,
            [
                "console-consumer",
                "--topic",
                self.topic,
                "--consumer.config",
                "tests/files/client.properties",
                "-c",
                self.cluster,
                "-n",
                self.namespace,
            ],
        )
        assert result.exit_code == 0

        native_command = (
            "bin/kafka-console-consumer.sh --bootstrap-server"
            " {cluster}-kafka-brokers:9093 --topic {topic}  --consumer.config"
            " /tmp/client.properties;rm -rf /tmp/truststore.jks;rm -rf /tmp/user.p12;rm"
            " -rf /tmp/client.properties;"
        )
        mock_os.system.assert_called_with(
            Kubectl()
            .exec("-it", "{cluster}-kafka-0")
            .container("kafka")
            .namespace(self.namespace)
            .exec_command(native_command)
            .build()
            .format(cluster=self.cluster, topic=self.topic)
        )

    @mock.patch("kfk.commands.console.os")
    def test_console_consumer_with_from_beginning(self, mock_os):
        from_beginning = True
        result = self.runner.invoke(
            kfk,
            [
                "console-consumer",
                "--topic",
                self.topic,
                "-c",
                self.cluster,
                "-n",
                self.namespace,
                "--from-beginning",
            ],
        )
        assert result.exit_code == 0

        native_command = (
            "bin/kafka-console-consumer.sh --bootstrap-server"
            " my-cluster-kafka-brokers:9092 --topic {topic} {from_beginning}"
        )
        mock_os.system.assert_called_with(
            Kubectl()
            .exec("-it", "{cluster}-kafka-0")
            .container("kafka")
            .namespace(self.namespace)
            .exec_command(native_command)
            .build()
            .format(
                cluster=self.cluster,
                topic=self.topic,
                from_beginning=(from_beginning and "--from-beginning" or ""),
            )
        )

    @mock.patch("kfk.commands.console.os")
    def test_console_producer(self, mock_os):
        result = self.runner.invoke(
            kfk,
            [
                "console-producer",
                "--topic",
                self.topic,
                "-c",
                self.cluster,
                "-n",
                self.namespace,
            ],
        )
        assert result.exit_code == 0
        native_command = (
            "bin/kafka-console-producer.sh --broker-list my-cluster-kafka-brokers:9092"
            " --topic {topic}"
        )
        mock_os.system.assert_called_with(
            Kubectl()
            .exec("-it", "{cluster}-kafka-0")
            .container("kafka")
            .namespace(self.namespace)
            .exec_command(native_command)
            .build()
            .format(cluster=self.cluster, topic=self.topic)
        )

    @mock.patch("kfk.commons.transfer_file_to_container")
    @mock.patch("kfk.commands.console.os")
    def test_console_producer_with_producer_config(
        self, mock_os, mock_transfer_file_to_container
    ):
        result = self.runner.invoke(
            kfk,
            [
                "console-producer",
                "--topic",
                self.topic,
                "--producer.config",
                "tests/files/client.properties",
                "-c",
                self.cluster,
                "-n",
                self.namespace,
            ],
        )
        assert result.exit_code == 0
        native_command = (
            "bin/kafka-console-producer.sh --broker-list {cluster}-kafka-brokers:9093"
            " --topic {topic} --producer.config /tmp/client.properties;rm -rf"
            " /tmp/truststore.jks;rm -rf /tmp/user.p12;rm -rf /tmp/client.properties;"
        )

        mock_os.system.assert_called_with(
            Kubectl()
            .exec("-it", "{cluster}-kafka-0")
            .container("kafka")
            .namespace(self.namespace)
            .exec_command(native_command)
            .build()
            .format(cluster=self.cluster, topic=self.topic)
        )



import os

import click
import yaml

from kfk.commands.main import kfk
from kfk.commons import (
    add_kv_config_to_resource,
    create_temp_file,
    delete_last_applied_configuration,
    delete_resource_config,
    get_resource_as_stream,
    open_file_in_system_editor,
    raise_exception_for_missing_options,
)
from kfk.config import STRIMZI_PATH, STRIMZI_VERSION
from kfk.kubectl_command_builder import Kubectl
from kfk.kubernetes_commons import create_using_yaml, delete_using_yaml
from kfk.messages import Messages
from kfk.option_extensions import NotRequiredIf


@click.option("-y", "--yes", "is_yes", help='"Yes" confirmation', is_flag=True)
@click.option(
    "-n",
    "--namespace",
    help="Namespace to use",
    required=True,
    cls=NotRequiredIf,
    options=["is_list"],
)
@click.option(
    "--delete-config",
    help="A cluster configuration override to be removed for an existing cluster",
    multiple=True,
)
@click.option(
    "--config",
    help="A cluster configuration override for the cluster being altered.",
    multiple=True,
)
@click.option("--alter", "is_alter", help="Alters the Kafka cluster.", is_flag=True)
@click.option("--delete", "is_delete", help="Deletes the Kafka cluster.", is_flag=True)
@click.option(
    "--zk-replicas", help="The number of zookeeper replicas for the cluster.", type=int
)
@click.option(
    "--replicas", help="The number of broker replicas for the cluster.", type=int
)
@click.option("--create", "is_create", help="Creates a Kafka cluster.", is_flag=True)
@click.option(
    "--describe",
    "is_describe",
    help="Lists details for the given cluster.",
    is_flag=True,
)
@click.option(
    "-o",
    "--output",
    help=(
        "Output format. One of:"
        " json|yaml|name|go-template|go-template-file|template|templatefile|jsonpath"
        "|jsonpath-file."
    ),
)
@click.option(
    "--list",
    "is_list",
    help="List all available clusters.",
    required=True,
    is_flag=True,
)
@click.option(
    "--cluster",
    help="Cluster Name",
    required=True,
    cls=NotRequiredIf,
    options=["is_list"],
)
@kfk.command()
def clusters(
    cluster,
    is_list,
    is_create,
    replicas,
    zk_replicas,
    is_describe,
    is_delete,
    is_alter,
    config,
    delete_config,
    output,
    namespace,
    is_yes,
):
    """Creates, alters, deletes, describes Kafka cluster(s)."""
    if is_list:
        list(namespace)
    elif is_create:
        create(cluster, replicas, zk_replicas, config, namespace, is_yes)
    elif is_describe:
        describe(cluster, output, namespace)
    elif is_delete:
        delete(cluster, namespace, is_yes)
    elif is_alter:
        alter(cluster, replicas, zk_replicas, config, delete_config, namespace)
    else:
        raise_exception_for_missing_options("clusters")


def list(namespace):
    os.system(Kubectl().get().kafkas().namespace(namespace).build())


def create(cluster, replicas, zk_replicas, config, namespace, is_yes):
    with open(
        "{strimzi_path}/examples/kafka/kafka-ephemeral.yaml".format(
            strimzi_path=STRIMZI_PATH
        ).format(version=STRIMZI_VERSION)
    ) as file:
        stream = file.read()

        cluster_dict = yaml.full_load(stream)

        cluster_dict["metadata"]["name"] = cluster

        _update_replicas(replicas, zk_replicas, cluster_dict)

        _add_config_if_provided(config, cluster_dict)

        cluster_yaml = yaml.dump(cluster_dict)
        cluster_temp_file = create_temp_file(cluster_yaml)

        if is_yes:
            is_confirmed = True
        else:
            open_file_in_system_editor(cluster_temp_file.name)
            is_confirmed = click.confirm(Messages.CLUSTER_CREATE_CONFIRMATION)
        if is_confirmed:
            create_using_yaml(cluster_temp_file.name, namespace)

        cluster_temp_file.close()


def describe(cluster, output, namespace):
    if output is not None:
        os.system(
            Kubectl().get().kafkas(cluster).namespace(namespace).output(output).build()
        )
    else:
        os.system(Kubectl().describe().kafkas(cluster).namespace(namespace).build())


def delete(cluster, namespace, is_yes):
    if is_yes:
        is_confirmed = True
    else:
        is_confirmed = click.confirm(Messages.CLUSTER_DELETE_CONFIRMATION)
    if is_confirmed:
        with open(
            "{strimzi_path}/examples/kafka/kafka-ephemeral.yaml".format(
                strimzi_path=STRIMZI_PATH
            ).format(version=STRIMZI_VERSION)
        ) as file:
            stream = file.read()

            cluster_dict = yaml.full_load(stream)

            cluster_dict["metadata"]["name"] = cluster

            cluster_yaml = yaml.dump(cluster_dict)
            cluster_temp_file = create_temp_file(cluster_yaml)

            delete_using_yaml(cluster_temp_file.name, namespace)

            cluster_temp_file.close()


def alter(cluster, replicas, zk_replicas, config, delete_config, namespace):
    if (
        len(config) > 0
        or len(delete_config) > 0
        or replicas is not None
        or zk_replicas is not None
    ):
        stream = get_resource_as_stream(
            "kafkas", resource_name=cluster, namespace=namespace
        )
        cluster_dict = yaml.full_load(stream)

        delete_last_applied_configuration(cluster_dict)

        _update_replicas(replicas, zk_replicas, cluster_dict)

        _add_config_if_provided(config, cluster_dict)

        if len(delete_config) > 0:
            if cluster_dict["spec"]["kafka"].get("config") is not None:
                delete_resource_config(
                    delete_config, cluster_dict["spec"]["kafka"]["config"]
                )

        cluster_yaml = yaml.dump(cluster_dict)
        cluster_temp_file = create_temp_file(cluster_yaml)
        os.system(
            Kubectl()
            .apply()
            .from_file("{cluster_temp_file_path}")
            .namespace(namespace)
            .build()
            .format(cluster_temp_file_path=cluster_temp_file.name)
        )
        cluster_temp_file.close()
    else:
        os.system(Kubectl().edit().kafkas(cluster).namespace(namespace).build())


def _update_replicas(replicas, zk_replicas, cluster_dict):
    if replicas is not None:
        cluster_dict["spec"]["kafka"]["replicas"] = int(replicas)
        min_insync_replicas = 1
        if replicas > 1:
            min_insync_replicas = replicas - 1
        cluster_dict["spec"]["kafka"]["config"][
            "offsets.topic.replication.factor"
        ] = int(replicas)
        cluster_dict["spec"]["kafka"]["config"][
            "transaction.state.log.replication.factor"
        ] = int(replicas)
        cluster_dict["spec"]["kafka"]["config"]["default.replication.factor"] = int(
            replicas
        )
        cluster_dict["spec"]["kafka"]["config"][
            "transaction.state.log.min.isr"
        ] = min_insync_replicas
        cluster_dict["spec"]["kafka"]["config"][
            "min.insync.replicas"
        ] = min_insync_replicas

    if zk_replicas is not None:
        cluster_dict["spec"]["zookeeper"]["replicas"] = int(zk_replicas)


def _add_config_if_provided(config, cluster_dict):
    if len(config) > 0:
        if cluster_dict["spec"]["kafka"].get("config") is None:
            cluster_dict["spec"]["kafka"]["config"] = {}
        add_kv_config_to_resource(config, cluster_dict["spec"]["kafka"]["config"])



from collections import namedtuple
from unittest import TestCase, mock

from kfk.config import _get_processor_type


def os_uname_mock(p_name_machine):
    OSUname = namedtuple("OSUname", ["machine"])
    return OSUname(machine=p_name_machine)


class TestKfkConfig(TestCase):
    @mock.patch("kfk.config._is_64_bit")
    @mock.patch("kfk.config.platform.uname")
    def test_processor_type_armhf_32bit(self, mock_os_uname, mock_is_64_bit):
        mock_os_uname.return_value = os_uname_mock("armhf")
        mock_is_64_bit.return_value = False
        assert _get_processor_type() == "arm"

    @mock.patch("kfk.config._is_64_bit")
    @mock.patch("kfk.config.platform.uname")
    def test_processor_type_armhf_64bit(self, mock_os_uname, mock_is_64_bit):
        mock_os_uname.return_value = os_uname_mock("armhf")
        mock_is_64_bit.return_value = True
        assert _get_processor_type() == "arm64"

    @mock.patch("kfk.config._is_64_bit")
    @mock.patch("kfk.config.platform.uname")
    def test_processor_type_aarch64_64bit(self, mock_os_uname, mock_is_64_bit):
        mock_os_uname.return_value = os_uname_mock("aarch64")
        mock_is_64_bit.return_value = True
        assert _get_processor_type() == "arm64"

    @mock.patch("kfk.config._is_64_bit")
    @mock.patch("kfk.config.os.uname")
    def test_processor_type_x86_64_64bit(self, mock_os_uname, mock_is_64_bit):
        mock_os_uname.return_value = os_uname_mock("x86_64")
        mock_is_64_bit.return_value = True
        assert _get_processor_type() == "amd64"

    @mock.patch("kfk.config._is_64_bit")
    @mock.patch("kfk.config.os.uname")
    def test_processor_type_x86_32bit(self, mock_os_uname, mock_is_64_bit):
        mock_os_uname.return_value = os_uname_mock("x86")
        mock_is_64_bit.return_value = False
        assert _get_processor_type() == "386"



import os

import click
import yaml

from kfk.commands.main import kfk
from kfk.commons import (
    add_kv_config_to_resource,
    apply_client_config_from_file,
    create_temp_file,
    delete_last_applied_configuration,
    delete_resource_config,
    get_resource_as_stream,
    raise_exception_for_missing_options,
)
from kfk.config import STRIMZI_PATH, STRIMZI_VERSION
from kfk.constants import KAFKA_PORT
from kfk.kubectl_command_builder import Kubectl
from kfk.kubernetes_commons import create_using_yaml, delete_using_yaml
from kfk.option_extensions import NotRequiredIf, RequiredIf


@click.option("-n", "--namespace", help="Namespace to use", required=True)
@click.option("-c", "--cluster", help="Cluster to use", required=True)
@click.option(
    "--delete-config",
    help="A topic configuration override to be removed for an existing topic",
    multiple=True,
)
@click.option(
    "--config",
    help="A topic configuration override for the topic being created or altered.",
    multiple=True,
)
@click.option(
    "--alter",
    "is_alter",
    help=(
        "Alter the number of partitions, replica assignment, and/or configuration of"
        " the topic."
    ),
    is_flag=True,
)
@click.option("--delete", "is_delete", help="Delete a topic.", is_flag=True)
@click.option(
    "--command-config",
    help=(
        "Property file containing configs to be config property file passed to Admin"
        " Client."
    ),
)
@click.option(
    "--native",
    help="List details for the given topic natively.",
    is_flag=True,
    cls=RequiredIf,
    options=["is_describe"],
)
@click.option(
    "-o",
    "--output",
    help=(
        "Output format. One of:"
        " json|yaml|name|go-template|go-template-file|template|templatefile|jsonpath"
        "|jsonpath-file."
    ),
)
@click.option(
    "--describe", "is_describe", help="List details for the given topic.", is_flag=True
)
@click.option(
    "--replication-factor",
    help="The replication factor for each partition in the topic being created.",
    cls=RequiredIf,
    options=["is_create"],
    type=int,
)
@click.option(
    "--partitions",
    help="The number of partitions for the topic being created or altered.",
    cls=RequiredIf,
    options=["is_create"],
    type=int,
)
@click.option("--create", "is_create", help="Create a new topic.", is_flag=True)
@click.option("--list", "is_list", help="List all available topics.", is_flag=True)
@click.option(
    "--topic", help="Topic Name", required=True, cls=NotRequiredIf, options=["is_list"]
)
@kfk.command()
def topics(
    topic,
    is_list,
    is_create,
    partitions,
    replication_factor,
    is_describe,
    output,
    native,
    command_config,
    is_delete,
    is_alter,
    config,
    delete_config,
    cluster,
    namespace,
):
    """Creates, alters, deletes, describes Kafka topic(s)."""
    if is_list:
        list(cluster, namespace)
    elif is_create:
        create(topic, partitions, replication_factor, config, cluster, namespace)
    elif is_describe:
        describe(topic, output, native, command_config, cluster, namespace)
    elif is_delete:
        delete(topic, cluster, namespace)
    elif is_alter:
        alter(
            topic,
            partitions,
            replication_factor,
            config,
            delete_config,
            cluster,
            namespace,
        )
    else:
        raise_exception_for_missing_options("topics")


def list(cluster, namespace):
    os.system(
        Kubectl()
        .get()
        .kafkatopics()
        .label("strimzi.io/cluster={cluster}")
        .namespace(namespace)
        .build()
        .format(cluster=cluster)
    )


def create(topic, partitions, replication_factor, config, cluster, namespace):
    with open(
        "{strimzi_path}/examples/topic/kafka-topic.yaml".format(
            strimzi_path=STRIMZI_PATH
        ).format(version=STRIMZI_VERSION)
    ) as file:
        topic_dict = yaml.full_load(file)

        topic_dict["metadata"]["name"] = topic
        topic_dict["metadata"]["labels"]["strimzi.io/cluster"] = cluster
        topic_dict["spec"]["partitions"] = int(partitions)
        topic_dict["spec"]["replicas"] = int(replication_factor)

        _add_config_if_provided(config, topic_dict)

        topic_yaml = yaml.dump(topic_dict)
        topic_temp_file = create_temp_file(topic_yaml)

        create_using_yaml(topic_temp_file.name, namespace)

        topic_temp_file.close()


def describe(topic, output, native, command_config, cluster, namespace):
    if output is not None:
        os.system(
            Kubectl()
            .get()
            .kafkatopics(topic)
            .namespace(namespace)
            .output(output)
            .build()
        )
    else:
        if native:
            native_command = (
                "bin/kafka-topics.sh --bootstrap-server {cluster}-kafka-brokers:{port}"
                " --describe --topic {topic}"
            )
            pod = cluster + "-kafka-0"
            container = "kafka"
            if command_config is not None:
                native_command = apply_client_config_from_file(
                    native_command,
                    command_config,
                    "--command-config",
                    container,
                    pod,
                    namespace,
                )
            os.system(
                Kubectl()
                .exec("-it", pod)
                .container(container)
                .namespace(namespace)
                .exec_command(native_command)
                .build()
                .format(port=KAFKA_PORT, topic=topic, cluster=cluster)
            )
        else:
            os.system(
                Kubectl().describe().kafkatopics(topic).namespace(namespace).build()
            )


def delete(topic, cluster, namespace):
    with open(
        "{strimzi_path}/examples/topic/kafka-topic.yaml".format(
            strimzi_path=STRIMZI_PATH
        ).format(version=STRIMZI_VERSION)
    ) as file:
        topic_dict = yaml.full_load(file)

        topic_dict["metadata"]["name"] = topic
        topic_dict["metadata"]["labels"]["strimzi.io/cluster"] = cluster

        topic_yaml = yaml.dump(topic_dict)
        topic_temp_file = create_temp_file(topic_yaml)

        delete_using_yaml(topic_temp_file.name, namespace)

        topic_temp_file.close()


def alter(
    topic, partitions, replication_factor, config, delete_config, cluster, namespace
):
    stream = get_resource_as_stream(
        "kafkatopics", resource_name=topic, namespace=namespace
    )
    topic_dict = yaml.full_load(stream)

    delete_last_applied_configuration(topic_dict)

    if partitions is not None:
        topic_dict["spec"]["partitions"] = int(partitions)

    if replication_factor is not None:
        topic_dict["spec"]["replicas"] = int(replication_factor)

    _add_config_if_provided(config, topic_dict)

    if len(delete_config) > 0:
        if topic_dict["spec"].get("config") is not None:
            delete_resource_config(delete_config, topic_dict["spec"]["config"])

    topic_yaml = yaml.dump(topic_dict)
    topic_temp_file = create_temp_file(topic_yaml)

    os.system(
        Kubectl()
        .apply()
        .from_file("{topic_temp_file_path}")
        .namespace(namespace)
        .build()
        .format(topic_temp_file_path=topic_temp_file.name)
    )

    topic_temp_file.close()


def _add_config_if_provided(config, topic_dict):
    if len(config) > 0:
        if topic_dict["spec"].get("config") is None:
            topic_dict["spec"]["config"] = {}
        add_kv_config_to_resource(config, topic_dict["spec"]["config"])



from unittest import TestCase

from kfk.utils import convert_string_to_type, is_valid_url, snake_to_camel_case


class TestUtils(TestCase):
    def test_convert_string_to_int(self):
        val_str = "12"
        self.assertEqual(convert_string_to_type(val_str), 12)

    def test_convert_string_to_float(self):
        val_str = "12.5"
        self.assertEqual(convert_string_to_type(val_str), 12.5)

    def test_convert_string_to_boolean(self):
        val_str = "true"
        self.assertEqual(convert_string_to_type(val_str), True)
        val_str = "false"
        self.assertEqual(convert_string_to_type(val_str), False)

    def test_convert_string_to_str(self):
        val_str = "test"
        self.assertEqual(convert_string_to_type(val_str), "test")

    def test_snake_to_camel_case(self):
        val_str = "this_is_the_test_string"
        self.assertEqual(snake_to_camel_case(val_str), "thisIsTheTestString")

    def test_is_valid_url(self):
        not_valid_url_str = "not a valid url"
        valid_url_str = "https://www.systemcraftsman.com"
        not_valid_url_str2 = "systemcraftsman.com"
        not_valid_url_str3 = "systemcraftsman.com"

        self.assertFalse(is_valid_url(not_valid_url_str))
        self.assertTrue(is_valid_url(valid_url_str))
        self.assertFalse(is_valid_url(not_valid_url_str2))
        self.assertFalse(is_valid_url(not_valid_url_str3))



import os

import click

from kfk.commands.main import kfk
from kfk.commons import apply_client_config_from_file
from kfk.constants import KAFKA_PORT
from kfk.kubectl_command_builder import Kubectl


@click.option("-n", "--namespace", help="Namespace to use", required=True)
@click.option("-c", "--cluster", help="Cluster to use", required=True)
@click.option("--from-beginning", help="Consumes messages from beginning", is_flag=True)
@click.option(
    "--consumer.config", "consumer_config", help="Consumer config properties file."
)
@click.option("--topic", help="Topic Name", required=True)
@kfk.command()
def console_consumer(topic, consumer_config, from_beginning, cluster, namespace):
    """Reads data from Kafka topics and outputs it to standard output."""
    native_command = (
        "bin/kafka-console-consumer.sh --bootstrap-server"
        " {cluster}-kafka-brokers:{port} --topic {topic} {from_beginning}"
    )
    pod = cluster + "-kafka-0"
    container = "kafka"
    if consumer_config is not None:
        native_command = apply_client_config_from_file(
            native_command,
            consumer_config,
            "--consumer.config",
            container,
            pod,
            namespace,
        )
    os.system(
        Kubectl()
        .exec("-it", pod)
        .container(container)
        .namespace(namespace)
        .exec_command(native_command)
        .build()
        .format(
            port=KAFKA_PORT,
            topic=topic,
            cluster=cluster,
            from_beginning=(from_beginning and "--from-beginning" or ""),
        )
    )


@click.option("-n", "--namespace", help="Namespace to use", required=True)
@click.option("-c", "--cluster", help="Cluster to use", required=True)
@click.option(
    "--producer.config", "producer_config", help="Producer config properties file."
)
@click.option("--topic", help="Topic Name", required=True)
@kfk.command()
def console_producer(topic, producer_config, cluster, namespace):
    """Reads data from standard input and publish it to Kafka."""
    native_command = (
        "bin/kafka-console-producer.sh --broker-list {cluster}-kafka-brokers:{port}"
        " --topic {topic}"
    )
    pod = cluster + "-kafka-0"
    container = "kafka"
    if producer_config is not None:
        native_command = apply_client_config_from_file(
            native_command,
            producer_config,
            "--producer.config",
            container,
            pod,
            namespace,
        )
    os.system(
        Kubectl()
        .exec("-it", pod)
        .container(container)
        .namespace(namespace)
        .exec_command(native_command)
        .build()
        .format(port=KAFKA_PORT, topic=topic, cluster=cluster)
    )



import os

import click

from kfk.commands.main import kfk


@kfk.command()
def env():
    """Prints the environment variable values for Strimzi Kafka CLI."""

    click.echo(
        "STRIMZI_KAFKA_CLI_BASE_PATH: {}".format(
            os.environ.get("STRIMZI_KAFKA_CLI_BASE_PATH")
        )
    )
    click.echo(
        "STRIMZI_KAFKA_CLI_STRIMZI_VERSION: {}".format(
            os.environ.get("STRIMZI_KAFKA_CLI_STRIMZI_VERSION")
        )
    )
    click.echo(
        "STRIMZI_KAFKA_CLI_STRIMZI_PATH: {}".format(
            os.environ.get("STRIMZI_KAFKA_CLI_STRIMZI_PATH")
        )
    )
    click.echo(
        "STRIMZI_KAFKA_CLI_KUBECTL_VERSION: {}".format(
            os.environ.get("STRIMZI_KAFKA_CLI_KUBECTL_VERSION")
        )
    )
    click.echo(
        "STRIMZI_KAFKA_CLI_KUBECTL_PATH: {}".format(
            os.environ.get("STRIMZI_KAFKA_CLI_KUBECTL_PATH")
        )
    )



import os

import click
import yaml

from kfk.commands.main import kfk
from kfk.commons import (
    add_kv_config_to_resource,
    create_temp_file,
    delete_last_applied_configuration,
    delete_resource_config,
    get_resource_as_stream,
    raise_exception_for_missing_options,
)
from kfk.config import STRIMZI_PATH, STRIMZI_VERSION
from kfk.kubectl_command_builder import Kubectl
from kfk.kubernetes_commons import create_using_yaml, delete_using_yaml
from kfk.option_extensions import NotRequiredIf, RequiredIf
from kfk.utils import snake_to_camel_case


@click.option("-n", "--namespace", help="Namespace to use", required=True)
@click.option("-c", "--cluster", help="Cluster to use", required=True)
@click.option(
    "--delete-quota",
    "delete_quota_tuple",
    help="User quotas to be removed.",
    multiple=True,
)
@click.option(
    "--quota",
    "quota_tuple",
    help="User's network and CPU utilization quotas in the Kafka cluster.",
    multiple=True,
)
@click.option(
    "--resource-pattern-type",
    help=(
        "The type of the resource pattern or <ANY|MATCH|LITERAL|PREFIXED> pattern"
        " filter. When adding acls, this should be a specific pattern type, e.g."
        " 'literal' or 'prefixed'. (default: literal)"
    ),
    default="literal",
)
@click.option(
    "--resource-name",
    help="ACL resource name.",
    cls=RequiredIf,
    options=["add_acl", "delete_acl"],
)
@click.option(
    "--resource-type",
    help="ACL resource type.",
    type=click.Choice(["topic", "group", "cluster"], case_sensitive=True),
    cls=RequiredIf,
    options=["add_acl", "delete_acl"],
)
@click.option(
    "--type",
    help="Operation type for ACL. (default: allow)",
    type=click.Choice(["allow", "deny"], case_sensitive=True),
    default="allow",
)
@click.option(
    "--host", help="Host which User will have access. (default: *)", default="*"
)
@click.option(
    "--operation",
    "operation_tuple",
    help="Operation that is being allowed or denied. (default: All)",
    default=["All"],
    multiple=True,
)
@click.option("--delete-acl", help="Delete ACL of User", is_flag=True)
@click.option("--add-acl", help="Add ACL to User", is_flag=True)
@click.option(
    "--authorization-type",
    help="Authorization type for user",
    type=click.Choice(["none", "simple"], case_sensitive=True),
)
@click.option(
    "--alter",
    "is_alter",
    help="Alter authentication-type, quotas, etc. of the user.",
    is_flag=True,
)
@click.option("--delete", "is_delete", help="Delete a user.", is_flag=True)
@click.option(
    "-o",
    "--output",
    help=(
        "Output format. One of:"
        " json|yaml|name|go-template|go-template-file|template|templatefile|jsonpath"
        "|jsonpath-file."
    ),
)
@click.option(
    "--describe", "is_describe", help="List details for the given user.", is_flag=True
)
@click.option(
    "--authentication-type",
    help="Authentication type for user",
    type=click.Choice(["tls", "scram-sha-512"], case_sensitive=True),
    cls=RequiredIf,
    options=["is_create"],
)
@click.option("--create", "is_create", help="Create a new user.", is_flag=True)
@click.option("--list", "is_list", help="List all available users.", is_flag=True)
@click.option(
    "--user", help="User Name", required=True, cls=NotRequiredIf, options=["is_list"]
)
@kfk.command()
def users(
    user,
    is_list,
    is_create,
    authentication_type,
    is_describe,
    output,
    is_delete,
    is_alter,
    authorization_type,
    add_acl,
    delete_acl,
    operation_tuple,
    host,
    type,
    resource_type,
    resource_name,
    resource_pattern_type,
    quota_tuple,
    delete_quota_tuple,
    cluster,
    namespace,
):
    """Creates, alters, deletes, describes Kafka users(s)."""
    if is_list:
        list(cluster, namespace)
    elif is_create:
        create(user, authentication_type, quota_tuple, cluster, namespace)
    elif is_describe:
        describe(user, output, cluster, namespace)
    elif is_delete:
        delete(user, cluster, namespace)
    elif is_alter:
        alter(
            user,
            authentication_type,
            authorization_type,
            add_acl,
            delete_acl,
            operation_tuple,
            host,
            type,
            resource_type,
            resource_name,
            resource_pattern_type,
            quota_tuple,
            delete_quota_tuple,
            cluster,
            namespace,
        )
    else:
        raise_exception_for_missing_options("users")


def list(cluster, namespace):
    os.system(
        Kubectl()
        .get()
        .kafkausers()
        .label("strimzi.io/cluster={cluster}")
        .namespace(namespace)
        .build()
        .format(cluster=cluster)
    )


def create(user, authentication_type, quota_tuple, cluster, namespace):
    with open(
        "{strimzi_path}/examples/user/kafka-user.yaml".format(
            strimzi_path=STRIMZI_PATH
        ).format(version=STRIMZI_VERSION)
    ) as file:
        user_dict = yaml.full_load(file)

        user_dict["metadata"]["name"] = user
        user_dict["metadata"]["labels"]["strimzi.io/cluster"] = cluster

        user_dict["spec"]["authentication"]["type"] = authentication_type
        user_dict["spec"].pop("authorization")

        if len(quota_tuple) > 0:
            if user_dict["spec"].get("quotas") is None:
                user_dict["spec"]["quotas"] = {}
            add_kv_config_to_resource(quota_tuple, user_dict["spec"]["quotas"])

        user_yaml = yaml.dump(user_dict)
        user_temp_file = create_temp_file(user_yaml)

        create_using_yaml(user_temp_file.name, namespace)

        user_temp_file.close()


def describe(user, output, cluster, namespace):
    if output is not None:
        os.system(
            Kubectl().get().kafkausers(user).namespace(namespace).output(output).build()
        )
    else:
        os.system(Kubectl().describe().kafkausers(user).namespace(namespace).build())


def delete(user, cluster, namespace):
    with open(
        "{strimzi_path}/examples/user/kafka-user.yaml".format(
            strimzi_path=STRIMZI_PATH
        ).format(version=STRIMZI_VERSION)
    ) as file:
        user_dict = yaml.full_load(file)

        user_dict["metadata"]["name"] = user
        user_dict["metadata"]["labels"]["strimzi.io/cluster"] = cluster

        user_yaml = yaml.dump(user_dict)
        user_temp_file = create_temp_file(user_yaml)

        delete_using_yaml(user_temp_file.name, namespace)

        user_temp_file.close()


def alter(
    user,
    authentication_type,
    authorization_type,
    add_acl,
    delete_acl,
    operation_tuple,
    host,
    type,
    resource_type,
    resource_name,
    resource_pattern_type,
    quota_tuple,
    delete_quota_tuple,
    cluster,
    namespace,
):
    stream = get_resource_as_stream(
        "kafkausers", resource_name=user, namespace=namespace
    )
    user_dict = yaml.full_load(stream)

    delete_last_applied_configuration(user_dict)

    if authentication_type is not None:
        user_dict["spec"]["authentication"]["type"] = authentication_type

    if authorization_type is not None:
        if user_dict["spec"].get("authorization") is None:
            user_dict["spec"]["authorization"] = {}
        if authorization_type != "none":
            user_dict["spec"]["authorization"]["type"] = authorization_type
        else:
            user_dict["spec"].pop("authorization")

    if add_acl:
        if user_dict["spec"].get("authorization") is not None:
            _add_acl_option(
                user_dict,
                operation_tuple,
                host,
                type,
                resource_type,
                resource_name,
                resource_pattern_type,
            )

    if delete_acl:
        if user_dict["spec"].get("authorization") is not None:
            _delete_acl_option(
                user_dict,
                operation_tuple,
                host,
                type,
                resource_type,
                resource_name,
                resource_pattern_type,
            )

    if len(quota_tuple) > 0:
        if user_dict["spec"].get("quotas") is None:
            user_dict["spec"]["quotas"] = {}
        add_kv_config_to_resource(
            quota_tuple, user_dict["spec"]["quotas"], snake_to_camel_case
        )

    if len(delete_quota_tuple) > 0:
        if user_dict["spec"].get("quotas") is not None:
            delete_resource_config(
                delete_quota_tuple, user_dict["spec"]["quotas"], snake_to_camel_case
            )

    user_yaml = yaml.dump(user_dict)
    user_temp_file = create_temp_file(user_yaml)

    os.system(
        Kubectl()
        .apply()
        .from_file("{user_temp_file_path}")
        .namespace(namespace)
        .build()
        .format(user_temp_file_path=user_temp_file.name)
    )

    user_temp_file.close()


def _add_acl_option(
    user_dict,
    operation,
    host,
    type,
    resource_type,
    resource_name,
    resource_pattern_type,
):
    if user_dict["spec"]["authorization"].get("acls") is None:
        user_dict["spec"]["authorization"]["acls"] = []
    for operation_str in operation:
        acl_dict = {
            "operation": operation_str,
            "host": host,
            "type": type,
            "resource": {
                "type": resource_type,
                "name": resource_name,
                "patternType": resource_pattern_type,
            },
        }
        user_dict["spec"]["authorization"]["acls"].append(acl_dict)


def _delete_acl_option(
    user_dict,
    operation,
    host,
    type,
    resource_type,
    resource_name,
    resource_pattern_type,
):
    if user_dict["spec"]["authorization"].get("acls") is None:
        user_dict["spec"]["authorization"]["acls"] = []
    for operation_str in operation:
        for acl_dict in user_dict["spec"]["authorization"]["acls"]:
            acl_dict_to_be_deleted = {
                "operation": operation_str,
                "host": host,
                "type": type,
                "resource": {
                    "type": resource_type,
                    "name": resource_name,
                    "patternType": resource_pattern_type,
                },
            }
            if acl_dict == acl_dict_to_be_deleted:
                if len(user_dict["spec"]["authorization"]["acls"]) == 1:
                    user_dict["spec"].pop("authorization")
                else:
                    user_dict["spec"]["authorization"]["acls"].remove(acl_dict)



import io
import ntpath
import os
import tempfile
from subprocess import call

import click
from jproperties import Properties

from kfk.constants import (
    COMMA,
    EQUALS,
    KAFKA_PORT,
    KAFKA_SECURE_PORT,
    KAFKA_SSL,
    NEW_LINE,
    SEMICOLON,
    SPACE,
)
from kfk.kubectl_command_builder import Kubectl
from kfk.utils import convert_string_to_type, get_list_by_split_string

# TODO: Message string to messages.py


def raise_exception_for_missing_options(command_str):
    raise click.ClickException(
        f"Missing options: kfk {command_str} [OPTIONS] \nTry 'kfk {command_str} --help'"
        " for help."
    )


def print_cluster_resource_not_found_msg(cluster, namespace):
    click.echo(f"No resource found in Kafka cluster: {cluster}, namespace: {namespace}")


def print_resource_not_found_msg(namespace):
    click.echo(f"No resource found in namespace: {namespace}")


def delete_last_applied_configuration(resource_dict):
    if "annotations" in resource_dict["metadata"]:
        resource_dict["metadata"]["annotations"].pop(
            "kubectl.kubernetes.io/last-applied-configuration", None
        )


def add_kv_config_to_resource(config, dict_part, *converters):
    if type(config) is tuple or type(config) is list:
        for config_str in config:
            for converter in converters:
                config_str = converter(config_str)
            config_list = get_kv_config_list(config_str)
            dict_part[config_list[0]] = convert_string_to_type(config_list[1])
    else:
        for converter in converters:
            config = converter(config)
        config_list = get_kv_config_list(config)
        dict_part[config_list[0]] = convert_string_to_type(config_list[1])


def add_properties_config_to_resource(properties, dict_part, *converters):
    for property_item in properties.items():
        for converter in converters:
            property_item = converter(property_item)
            if property_item is not None:
                dict_part[property_item[0]] = convert_string_to_type(
                    property_item[1].data
                )


def get_kv_config_list(config_str):
    return get_list_by_split_string(config_str, EQUALS)


def get_config_list(config_str):
    if config_str is None:
        return list()
    else:
        return get_list_by_split_string(config_str, COMMA)


def delete_resource_config(config, dict_part, *converters):
    if type(config) is tuple or type(config) is list:
        for config_str in config:
            for converter in converters:
                config_str = converter(config_str)
            dict_part.pop(config_str, None)
    else:
        dict_part.pop(config, None)


def resource_exists(
    resource_type=None, resource_name=None, cluster=None, namespace=None
):
    command = Kubectl().get().resource(resource_type).namespace(namespace)

    if cluster is not None:
        command = command.label(f"strimzi.io/cluster={cluster}")

    return resource_name in os.popen(command.build()).read()


def get_resource_yaml(
    resource_type=None, resource_name=None, cluster=None, namespace=None
):
    command = (
        Kubectl()
        .get()
        .resource(resource_type, resource_name)
        .namespace(namespace)
        .output("yaml")
    )

    if cluster is not None:
        command = command.label(f"strimzi.io/cluster={cluster}")

    return os.popen(command.build()).read()


def get_resource_as_stream(
    resource_type=None, resource_name=None, cluster=None, namespace=None
):
    resource_yaml = get_resource_yaml(resource_type, resource_name, cluster, namespace)

    if not resource_yaml:
        raise click.exceptions.Exit(1)

    return io.StringIO(resource_yaml)


def create_temp_file(stream):
    temp_file = tempfile.NamedTemporaryFile(mode="w+")
    temp_file.write(stream)
    temp_file.flush()
    return temp_file


def open_file_in_system_editor(file):
    call([os.environ.get("EDITOR", "vim"), file])


def transfer_file_to_container(
    source_file_path, dest_file_path, container, pod, namespace
):
    os.system(
        Kubectl()
        .cp(source_file_path, f"{namespace}/{pod}:" + dest_file_path)
        .container(container)
        .build()
    )


def apply_client_config_from_file(
    native_command, config_file_path, config_file_flag, container, pod, namespace
):
    port = KAFKA_PORT
    delete_file_command = ""
    with open(config_file_path) as file:
        temp_file = create_temp_file(file.read())
        lines = []
        with open(temp_file.name) as temp_file:
            for cnt, producer_property in enumerate(temp_file):
                producer_property = producer_property.strip()
                if "security.protocol" in producer_property:
                    producer_property_list = get_kv_config_list(producer_property)
                    if producer_property_list[1] == KAFKA_SSL:
                        port = KAFKA_SECURE_PORT
                if (
                    "ssl.truststore.location" in producer_property
                    or "ssl.keystore.location" in producer_property
                ):
                    producer_property_list = get_kv_config_list(producer_property)
                    file_path = producer_property_list[1]
                    new_file_path = "/tmp/" + ntpath.basename(file_path)
                    transfer_file_to_container(
                        file_path, new_file_path, container, pod, namespace
                    )
                    producer_property = producer_property_list[0] + "=" + new_file_path
                    delete_file_command = (
                        delete_file_command
                        + "rm -rf"
                        + SPACE
                        + new_file_path
                        + SEMICOLON
                    )
                lines.append(producer_property)
        with open(temp_file.name, "w") as temp_file:
            for line in lines:
                temp_file.write(line + NEW_LINE)
        new_config_file_path = "/tmp/" + ntpath.basename(config_file_path)
        transfer_file_to_container(
            temp_file.name, new_config_file_path, container, pod, namespace
        )
        delete_file_command = (
            delete_file_command + "rm -rf" + SPACE + new_config_file_path + SEMICOLON
        )
        native_command = (
            native_command + SPACE + config_file_flag + SPACE + new_config_file_path
        )
        temp_file.close()
    return (
        native_command.format_map(SafeDict(port=port)) + SEMICOLON + delete_file_command
    )


def get_properties_from_file(file):
    properties = Properties()
    properties.load(file.read())
    return properties


class SafeDict(dict):
    def __missing__(self, key):
        return "{" + key + "}"



import click


class NotRequiredIf(click.Option):
    # TODO: Refactor here

    def __init__(self, *args, **kwargs):
        self.options = kwargs.pop("options")
        assert self.options, "'options' parameter required"
        kwargs["help"] = (
            kwargs.get("help", "")
            + " This argument is mutually exclusive with %s" % self.options
        ).strip()
        super(NotRequiredIf, self).__init__(*args, **kwargs)

    def handle_parse_result(self, ctx, opts, args):
        control_options_exist = False

        for options in self.options:
            control_options_exist = options in opts
            if control_options_exist is True:
                break

        if control_options_exist:
            self.required = None

        return super(NotRequiredIf, self).handle_parse_result(ctx, opts, args)


class RequiredIf(click.Option):
    # TODO: Refactor here

    def __init__(self, *args, **kwargs):
        self.options = kwargs.pop("options")
        assert self.options, "'options' parameter required"
        kwargs["help"] = (
            kwargs.get("help", "")
            + " This argument is mutually inclusive with %s" % self.options
        ).strip()
        super(RequiredIf, self).__init__(*args, **kwargs)

    def handle_parse_result(self, ctx, opts, args):
        control_options_exist = False

        for options in self.options:
            control_options_exist = options in opts
            if control_options_exist is True:
                break

        if control_options_exist:
            self.required = True
        else:
            self.required = None

        return super(RequiredIf, self).handle_parse_result(ctx, opts, args)



from unittest import TestCase, mock

import pkg_resources
from click.testing import CliRunner

from kfk.config import KUBECTL_VERSION, STRIMZI_VERSION


class TestKfk(TestCase):
    def setUp(self):
        self.runner = CliRunner()

    @mock.patch("kfk.setup.setup")
    def test_kfk(self, mock_setup):
        from kfk.main import kfk

        result = self.runner.invoke(kfk)
        assert result.exit_code == 0
        mock_setup.assert_called()

    @mock.patch("kfk.setup.setup")
    def test_kfk_version(self, mock_setup):
        from kfk.main import kfk

        result = self.runner.invoke(kfk, ["--version"])
        assert result.exit_code == 0
        cli_version = pkg_resources.require("strimzi-kafka-cli")[0].version
        expected_version = f"""CLI Version: {cli_version}
Strimzi Version: {STRIMZI_VERSION}
Kubectl Version: {KUBECTL_VERSION}
"""

        assert result.output == expected_version



from unittest import TestCase, mock

import click
from click import ClickException
from click.testing import CliRunner

from kfk.commands.connect.clusters import connect
from kfk.kubectl_command_builder import Kubectl


class TestKfkConnect(TestCase):
    def setUp(self):
        self.runner = CliRunner()
        self.cluster = "my-connect-cluster"
        self.namespace = "kafka"
        self.connect_config_file = "tests/files/connect.properties"
        self.connector_config_file_1 = "tests/files/twitter-connector.properties"
        self.connector_config_file_2 = "tests/files/file-stream-connector.properties"
        self.registry_userpass = "someuserpass"
        self.registry_server = "quay.io/systemcraftsman/test-connect-cluster:latest"
        self.push_secret_name = f"{self.cluster}-push-secret"

    def test_no_option(self):
        result = self.runner.invoke(
            connect, ["clusters", "--cluster", self.cluster, "-n", self.namespace]
        )
        assert result.exit_code == 1
        assert "Missing options: kfk connect" in result.output

    def test_create_custer_without_config_file(self):
        result = self.runner.invoke(
            connect,
            ["clusters", "--create", "--cluster", self.cluster, "-n", self.namespace],
        )

        assert result.exit_code == 2

    @mock.patch("kfk.commands.connect.clusters.click.prompt")
    @mock.patch("kfk.commands.connect.clusters.create_temp_file")
    @mock.patch("kfk.commands.connect.clusters.open_file_in_system_editor")
    @mock.patch("kfk.commands.connect.clusters.click.confirm")
    @mock.patch("kfk.commands.connect.clusters.create_registry_secret")
    @mock.patch("kfk.commands.connect.clusters.create_using_yaml")
    def test_create_cluster(
        self,
        mock_create_using_yaml,
        mock_create_registry_secret,
        mock_click_confirm,
        mock_open_file_in_system_editor,
        mock_create_temp_file,
        mock_click_prompt,
    ):
        mock_click_confirm.return_value = True
        mock_click_prompt.return_value = self.registry_userpass

        result = self.runner.invoke(
            connect,
            [
                "clusters",
                "--create",
                "--cluster",
                self.cluster,
                self.connect_config_file,
                "-n",
                self.namespace,
            ],
        )
        assert result.exit_code == 0

        result_connect_yaml = mock_create_temp_file.call_args[0][0]

        with open("tests/files/yaml/kafka-connect.yaml") as file:
            expected_connect_yaml = file.read()

            assert expected_connect_yaml == result_connect_yaml

        mock_create_registry_secret.assert_called_once()
        mock_create_using_yaml.assert_called_once()

    @mock.patch("kfk.commands.connect.clusters.click.prompt")
    @mock.patch("kfk.commands.connect.clusters.create_temp_file")
    @mock.patch("kfk.commands.connect.clusters.open_file_in_system_editor")
    @mock.patch("kfk.commands.connect.clusters.click.confirm")
    @mock.patch("kfk.commands.connect.clusters.create_registry_secret")
    @mock.patch("kfk.commands.connect.clusters.create_using_yaml")
    def test_create_cluster_with_image(
        self,
        mock_create_using_yaml,
        mock_create_registry_secret,
        mock_click_confirm,
        mock_open_file_in_system_editor,
        mock_create_temp_file,
        mock_click_prompt,
    ):
        mock_click_confirm.return_value = True
        mock_click_prompt.return_value = self.registry_userpass

        result = self.runner.invoke(
            connect,
            [
                "clusters",
                "--create",
                "--cluster",
                self.cluster,
                "tests/files/connect_with_only_image.properties",
                "-n",
                self.namespace,
            ],
        )
        assert result.exit_code == 0

        result_connect_yaml = mock_create_temp_file.call_args[0][0]

        with open("tests/files/yaml/kafka-connect_with_image.yaml") as file:
            expected_connect_yaml = file.read()

            assert expected_connect_yaml == result_connect_yaml

        mock_create_registry_secret.assert_not_called()
        mock_create_using_yaml.assert_called_once()

    @mock.patch("kfk.commands.connect.clusters.click.prompt")
    @mock.patch("kfk.commands.connect.clusters.create_temp_file")
    @mock.patch("kfk.commands.connect.clusters.open_file_in_system_editor")
    @mock.patch("kfk.commands.connect.clusters.click.confirm")
    @mock.patch("kfk.commands.connect.clusters.create_registry_secret")
    @mock.patch("kfk.commands.connect.clusters.create_using_yaml")
    def test_create_cluster_with_command_error(
        self,
        mock_create_using_yaml,
        mock_create_registry_secret,
        mock_click_confirm,
        mock_open_file_in_system_editor,
        mock_create_temp_file,
        mock_click_prompt,
    ):
        mock_click_confirm.return_value = True
        mock_click_prompt.return_value = self.registry_userpass
        mock_create_registry_secret.side_effect = ClickException

        result = self.runner.invoke(
            connect,
            [
                "clusters",
                "--create",
                "--cluster",
                self.cluster,
                self.connect_config_file,
                "-n",
                self.namespace,
            ],
        )
        assert result.exit_code == 1

        mock_create_registry_secret.assert_called_once()
        mock_create_using_yaml.assert_not_called()

    @mock.patch("kfk.commands.connect.clusters.click.prompt")
    @mock.patch("kfk.commands.connect.clusters.create_temp_file")
    @mock.patch("kfk.commands.connect.clusters.open_file_in_system_editor")
    @mock.patch("kfk.commands.connect.clusters.click.confirm")
    @mock.patch("kfk.commands.connect.clusters.create_registry_secret")
    @mock.patch("kfk.commands.connect.clusters.create_using_yaml")
    def test_create_cluster_with_zip_jar_plugins(
        self,
        mock_create_using_yaml,
        mock_create_registry_secret,
        mock_click_confirm,
        mock_open_file_in_system_editor,
        mock_create_temp_file,
        mock_click_prompt,
    ):
        mock_click_confirm.return_value = True
        mock_click_prompt.return_value = self.registry_userpass

        result = self.runner.invoke(
            connect,
            [
                "clusters",
                "--create",
                "--cluster",
                self.cluster,
                "tests/files/connect_with_zip_jar_plugins.properties",
                "-n",
                self.namespace,
            ],
        )
        assert result.exit_code == 0

        result_connect_yaml = mock_create_temp_file.call_args[0][0]

        with open("tests/files/yaml/kafka-connect_with_zip_jar_plugins.yaml") as file:
            expected_connect_yaml = file.read()

            assert expected_connect_yaml == result_connect_yaml

        mock_create_registry_secret.assert_called_once()
        mock_create_using_yaml.assert_called_once()

    @mock.patch("kfk.commands.connect.clusters.click.prompt")
    @mock.patch("kfk.commands.connect.clusters.create_temp_file")
    @mock.patch("kfk.commands.connect.clusters.open_file_in_system_editor")
    @mock.patch("kfk.commands.connect.clusters.click.confirm")
    @mock.patch("kfk.commands.connect.clusters.os")
    def test_create_cluster_with_invalid_url(
        self,
        mock_os,
        mock_click_confirm,
        mock_open_file_in_system_editor,
        mock_create_temp_file,
        mock_click_prompt,
    ):
        mock_click_confirm.return_value = True
        mock_click_prompt.return_value = self.registry_userpass

        result = self.runner.invoke(
            connect,
            [
                "clusters",
                "--create",
                "--cluster",
                self.cluster,
                "tests/files/connect_with_invalid_url.properties",
                "-n",
                self.namespace,
            ],
        )
        assert result.exit_code == 1

        self.assertRaises(click.ClickException)

    @mock.patch("kfk.commands.connect.clusters.click.prompt")
    @mock.patch("kfk.commands.connect.clusters.create_temp_file")
    @mock.patch("kfk.commands.connect.clusters.open_file_in_system_editor")
    @mock.patch("kfk.commands.connect.clusters.click.confirm")
    @mock.patch("kfk.commands.connect.clusters.create_registry_secret")
    @mock.patch("kfk.commands.connect.clusters.create_using_yaml")
    def test_create_cluster_with_yes_flag(
        self,
        mock_create_using_yaml,
        mock_create_registry_secret,
        mock_click_confirm,
        mock_open_file_in_system_editor,
        mock_create_temp_file,
        mock_click_prompt,
    ):
        mock_click_confirm.return_value = False
        mock_click_prompt.return_value = self.registry_userpass

        result = self.runner.invoke(
            connect,
            [
                "clusters",
                "--create",
                "--cluster",
                self.cluster,
                self.connect_config_file,
                "-n",
                self.namespace,
                "-y",
            ],
        )
        assert result.exit_code == 0

        with open("tests/files/yaml/kafka-connect.yaml") as file:
            expected_connect_yaml = file.read()
            result_connect_yaml = mock_create_temp_file.call_args[0][0]
            assert expected_connect_yaml == result_connect_yaml

        mock_create_registry_secret.assert_called_once()
        mock_create_using_yaml.assert_called_once()

    @mock.patch("kfk.commands.connect.clusters.click.prompt")
    @mock.patch("kfk.commands.connect.clusters.create_temp_file")
    @mock.patch("kfk.commands.connect.clusters.open_file_in_system_editor")
    @mock.patch("kfk.commands.connect.clusters.click.confirm")
    @mock.patch("kfk.commands.connect.clusters.create_registry_secret")
    @mock.patch("kfk.commands.connect.clusters.create_using_yaml")
    def test_create_cluster_three_replicas(
        self,
        mock_create_using_yaml,
        mock_create_registry_secret,
        mock_click_confirm,
        mock_open_file_in_system_editor,
        mock_create_temp_file,
        mock_click_prompt,
    ):
        mock_click_confirm.return_value = True
        mock_click_prompt.return_value = self.registry_userpass

        result = self.runner.invoke(
            connect,
            [
                "clusters",
                "--create",
                "--cluster",
                self.cluster,
                "--replicas",
                3,
                self.connect_config_file,
                "-n",
                self.namespace,
            ],
        )
        assert result.exit_code == 0

        with open("tests/files/yaml/kafka-connect_with_three_replicas.yaml") as file:
            expected_connect_yaml = file.read()
            result_connect_yaml = mock_create_temp_file.call_args[0][0]
            assert expected_connect_yaml == result_connect_yaml

        mock_create_registry_secret.assert_called_once()
        mock_create_using_yaml.assert_called_once()

    @mock.patch("kfk.commands.connect.connectors.create_temp_file")
    @mock.patch("kfk.commands.connect.connectors.create_using_yaml")
    @mock.patch("kfk.commands.connect.clusters.click.prompt")
    @mock.patch("kfk.commands.connect.clusters.create_temp_file")
    @mock.patch("kfk.commands.connect.clusters.open_file_in_system_editor")
    @mock.patch("kfk.commands.connect.clusters.click.confirm")
    @mock.patch("kfk.commands.connect.clusters.create_registry_secret")
    @mock.patch("kfk.commands.connect.clusters.create_using_yaml")
    def test_create_cluster_with_connector_config(
        self,
        mock_create_using_yaml,
        mock_create_registry_secret,
        mock_click_confirm,
        mock_open_file_in_system_editor,
        mock_create_temp_file,
        mock_click_prompt,
        mock_connectors_create_using_yaml,
        mock_create_temp_file_connectors,
    ):
        mock_click_confirm.return_value = True
        mock_click_prompt.return_value = self.registry_userpass

        result = self.runner.invoke(
            connect,
            [
                "clusters",
                "--create",
                "--cluster",
                self.cluster,
                self.connect_config_file,
                self.connector_config_file_1,
                "-n",
                self.namespace,
            ],
        )
        assert result.exit_code == 0

        with open("tests/files/yaml/kafka-connect.yaml") as file:
            expected_connect_yaml = file.read()
            result_connect_yaml = mock_create_temp_file.call_args[0][0]
            assert expected_connect_yaml == result_connect_yaml

        mock_create_registry_secret.assert_called_once()
        mock_create_using_yaml.assert_called_once()

        with open("tests/files/yaml/kafka-connect-connector-twitter.yaml") as file:
            expected_connector_yaml = file.read()
            result_connector_yaml = mock_create_temp_file_connectors.call_args[0][0]
            assert expected_connector_yaml == result_connector_yaml

        mock_connectors_create_using_yaml.assert_called_once()

    @mock.patch("kfk.commands.connect.connectors.create_temp_file")
    @mock.patch("kfk.commands.connect.connectors.create_using_yaml")
    @mock.patch("kfk.commands.connect.clusters.click.prompt")
    @mock.patch("kfk.commands.connect.clusters.create_temp_file")
    @mock.patch("kfk.commands.connect.clusters.open_file_in_system_editor")
    @mock.patch("kfk.commands.connect.clusters.click.confirm")
    @mock.patch("kfk.commands.connect.clusters.create_registry_secret")
    @mock.patch("kfk.commands.connect.clusters.create_using_yaml")
    def test_create_cluster_with_two_connectors_config(
        self,
        mock_create_using_yaml,
        mock_create_registry_secret,
        mock_click_confirm,
        mock_open_file_in_system_editor,
        mock_create_temp_file,
        mock_click_prompt,
        mock_connectors_create_using_yaml,
        mock_create_temp_file_connectors,
    ):
        mock_click_confirm.return_value = True
        mock_click_prompt.return_value = self.registry_userpass

        result = self.runner.invoke(
            connect,
            [
                "clusters",
                "--create",
                "--cluster",
                self.cluster,
                self.connect_config_file,
                self.connector_config_file_1,
                self.connector_config_file_2,
                "-n",
                self.namespace,
            ],
        )
        assert result.exit_code == 0

        with open("tests/files/yaml/kafka-connect.yaml") as file:
            expected_connect_yaml = file.read()
            result_connect_yaml = mock_create_temp_file.call_args[0][0]
            assert expected_connect_yaml == result_connect_yaml

        mock_create_registry_secret.assert_called_once()
        mock_create_using_yaml.assert_called_once()

        with open("tests/files/yaml/kafka-connect-connector-twitter.yaml") as file:
            expected_connector_yaml = file.read()
            result_connector_yaml = mock_create_temp_file_connectors.call_args_list[0][
                0
            ][0]
            assert expected_connector_yaml == result_connector_yaml

        with open("tests/files/yaml/kafka-connect-connector-file-stream.yaml") as file:
            expected_connector_yaml = file.read()
            result_connector_yaml = mock_create_temp_file_connectors.call_args_list[1][
                0
            ][0]
            assert expected_connector_yaml == result_connector_yaml

        assert mock_connectors_create_using_yaml.call_count == 2

    @mock.patch("kfk.commands.connect.clusters.os")
    def test_list_clusters(self, mock_os):
        result = self.runner.invoke(
            connect, ["clusters", "--list", "-n", self.namespace]
        )
        assert result.exit_code == 0
        mock_os.system.assert_called_with(
            Kubectl().get().kafkaconnects().namespace(self.namespace).build()
        )

    @mock.patch("kfk.commands.connect.clusters.os")
    def test_list_clusters_all_namespaces(self, mock_os):
        result = self.runner.invoke(connect, ["clusters", "--list"])
        assert result.exit_code == 0
        mock_os.system.assert_called_with(
            Kubectl().get().kafkaconnects().namespace().build()
        )

    @mock.patch("kfk.commands.connect.clusters.os")
    def test_describe_cluster(self, mock_os):
        result = self.runner.invoke(
            connect,
            ["clusters", "--describe", "--cluster", self.cluster, "-n", self.namespace],
        )
        assert result.exit_code == 0
        mock_os.system.assert_called_with(
            Kubectl()
            .describe()
            .kafkaconnects(self.cluster)
            .namespace(self.namespace)
            .build()
        )

    @mock.patch("kfk.commands.connect.clusters.os")
    def test_describe_cluster_output_yaml(self, mock_os):
        result = self.runner.invoke(
            connect,
            [
                "clusters",
                "--describe",
                "--cluster",
                self.cluster,
                "-n",
                self.namespace,
                "-o",
                "yaml",
            ],
        )
        assert result.exit_code == 0
        mock_os.system.assert_called_with(
            Kubectl()
            .get()
            .kafkaconnects(self.cluster)
            .namespace(self.namespace)
            .output("yaml")
            .build()
        )

    @mock.patch("kfk.commands.connect.clusters.click.confirm")
    @mock.patch("kfk.commands.connect.clusters.delete_object")
    @mock.patch("kfk.commands.connect.clusters.delete_using_yaml")
    def test_delete_cluster(
        self, mock_delete_using_yaml, mock_delete_object, mock_click_confirm
    ):
        mock_click_confirm.return_value = True
        result = self.runner.invoke(
            connect,
            ["clusters", "--delete", "--cluster", self.cluster, "-n", self.namespace],
        )
        assert result.exit_code == 0
        mock_delete_using_yaml.assert_called_once()
        mock_delete_object.assert_called_with(
            f"{self.cluster}-push-secret", "secret", self.namespace
        )

    @mock.patch("kfk.commands.connect.clusters.click.confirm")
    @mock.patch("kfk.commands.connect.clusters.delete_object")
    @mock.patch("kfk.commands.connect.clusters.delete_using_yaml")
    def test_delete_cluster_with_yes_flag(
        self, mock_delete_using_yaml, mock_delete_object, mock_click_confirm
    ):
        mock_click_confirm.return_value = False
        result = self.runner.invoke(
            connect,
            [
                "clusters",
                "--delete",
                "--cluster",
                self.cluster,
                "-n",
                self.namespace,
                "-y",
            ],
        )
        assert result.exit_code == 0

        mock_delete_using_yaml.assert_called_once()
        mock_delete_object.assert_called_with(
            f"{self.cluster}-push-secret", "secret", self.namespace
        )

    @mock.patch("kfk.commands.connect.clusters.os")
    def test_alter_cluster_without_parameters(self, mock_os):
        result = self.runner.invoke(
            connect,
            ["clusters", "--alter", "--cluster", self.cluster, "-n", self.namespace],
        )
        assert result.exit_code == 0
        mock_os.system.assert_called_with(
            Kubectl()
            .edit()
            .kafkaconnects(self.cluster)
            .namespace(self.namespace)
            .build()
        )

    @mock.patch("kfk.commands.connect.clusters.create_temp_file")
    @mock.patch("kfk.commons.get_resource_yaml")
    @mock.patch("kfk.commands.connect.clusters.os")
    def test_alter_cluster_with_replica_param(
        self, mock_os, mock_get_resource_yaml, mock_create_temp_file
    ):
        with open("tests/files/yaml/kafka-connect.yaml") as file:
            connect_yaml = file.read()

            mock_get_resource_yaml.return_value = connect_yaml

            result = self.runner.invoke(
                connect,
                [
                    "clusters",
                    "--alter",
                    "--cluster",
                    self.cluster,
                    "--replicas",
                    3,
                    "-n",
                    self.namespace,
                ],
            )
            assert result.exit_code == 0

            with open(
                "tests/files/yaml/kafka-connect_with_three_replicas.yaml"
            ) as file:
                expected_connect_yaml = file.read()
                result_connect_yaml = mock_create_temp_file.call_args[0][0]
                assert expected_connect_yaml == result_connect_yaml

    @mock.patch("kfk.commands.connect.clusters.create_temp_file")
    @mock.patch("kfk.commons.get_resource_yaml")
    @mock.patch("kfk.commands.connect.clusters.os")
    def test_alter_cluster_with_different_config_file(
        self, mock_os, mock_get_resource_yaml, mock_create_temp_file
    ):
        with open("tests/files/yaml/kafka-connect.yaml") as file:
            connect_yaml = file.read()

            mock_get_resource_yaml.return_value = connect_yaml

            result = self.runner.invoke(
                connect,
                [
                    "clusters",
                    "--alter",
                    "--cluster",
                    self.cluster,
                    "tests/files/connect_with_zip_jar_plugins.properties",
                    "-n",
                    self.namespace,
                ],
            )
            assert result.exit_code == 0

            with open(
                "tests/files/yaml/kafka-connect_with_zip_jar_plugins.yaml"
            ) as file:
                expected_connect_yaml = file.read()
                result_connect_yaml = mock_create_temp_file.call_args[0][0]
                assert expected_connect_yaml == result_connect_yaml

    @mock.patch("kfk.commands.connect.clusters.create_temp_file")
    @mock.patch("kfk.commons.get_resource_yaml")
    @mock.patch("kfk.commands.connect.clusters.os")
    def test_alter_cluster_with_only_image_config_file(
        self, mock_os, mock_get_resource_yaml, mock_create_temp_file
    ):
        with open("tests/files/yaml/kafka-connect.yaml") as file:
            connect_yaml = file.read()

            mock_get_resource_yaml.return_value = connect_yaml

            result = self.runner.invoke(
                connect,
                [
                    "clusters",
                    "--alter",
                    "--cluster",
                    self.cluster,
                    "tests/files/connect_with_only_image.properties",
                    "-n",
                    self.namespace,
                ],
            )
            assert result.exit_code == 0

            with open("tests/files/yaml/kafka-connect_with_image.yaml") as file:
                expected_connect_yaml = file.read()
                result_connect_yaml = mock_create_temp_file.call_args[0][0]
                assert expected_connect_yaml == result_connect_yaml



import os
import tempfile
from unittest import TestCase, mock

from click.testing import CliRunner

from kfk.config import KUBECTL, STRIMZI_VERSION
from kfk.setup import setup


class TestSetup(TestCase):
    def setUp(self):
        self.runner = CliRunner()

    @mock.patch("kfk.setup.os.rename")
    @mock.patch("kfk.setup.KUBECTL_PATH", tempfile.mkdtemp() + "/" + KUBECTL)
    @mock.patch("kfk.setup._download_kubectl")
    def test_main(self, mock_download_kubectl, mock_rename):
        setup()
        assert mock_download_kubectl.call_count == 1

    @mock.patch("kfk.setup.os.rename")
    @mock.patch("kfk.setup.KUBECTL_PATH", tempfile.mkdtemp() + "/" + KUBECTL)
    @mock.patch("kfk.setup._download_kubectl")
    def test_download_kubectl_if_not_exists(self, mock_download_kubectl, mock_rename):
        setup()
        assert mock_download_kubectl.call_count == 1

    @mock.patch("kfk.setup.os.rename")
    @mock.patch("kfk.setup.KUBECTL_VERSION", "x.x.x")
    @mock.patch("kfk.setup._download_kubectl")
    def test_update_kubectl_if_new_version_exists(
        self, mock_download_kubectl, mock_rename
    ):
        setup()
        assert mock_download_kubectl.call_count == 1

    @mock.patch("kfk.setup.os.rename")
    @mock.patch("kfk.setup.KUBECTL_VERSION", "x.x.x")
    @mock.patch("kfk.setup._download_kubectl")
    def test_update_kubectl_if_new_version_exists_with_custom_kubectl_path(
        self, mock_download_kubectl, mock_rename
    ):
        os.environ["STRIMZI_KAFKA_CLI_KUBECTL_PATH"] = tempfile.mkdtemp()
        setup()
        assert mock_download_kubectl.call_count == 0

    @mock.patch("kfk.setup.os.rename")
    @mock.patch("kfk.setup.KUBECTL_VERSION", "x.x.x")
    @mock.patch("kfk.setup._download_kubectl")
    def test_update_kubectl_if_new_version_exists_with_custom_kubectl_version(
        self, mock_download_kubectl, mock_rename
    ):
        os.environ["STRIMZI_KAFKA_CLI_KUBECTL_VERSION"] = "x.x.y"
        setup()
        assert mock_download_kubectl.call_count == 0

    @mock.patch("kfk.setup.os.rename")
    @mock.patch("kfk.setup.os.remove")
    @mock.patch("kfk.setup.tarfile")
    @mock.patch("kfk.setup.print")
    @mock.patch("kfk.setup.wget.download")
    @mock.patch("kfk.setup.STRIMZI_PATH", tempfile.mkdtemp() + "/strimzi-x.x.x")
    def test_download_strimzi_if_not_exists(
        self, mock_wget_download, mock_print, mock_tarfile, mock_remove, mock_rename
    ):
        setup()
        mock_print.assert_called_with(
            "Extracting Strimzi {version}...\n".format(version=STRIMZI_VERSION)
        )



from unittest import TestCase, mock

from click.testing import CliRunner

from kfk.commands.topics import kfk
from kfk.kubectl_command_builder import Kubectl


class TestKfkTopics(TestCase):
    def setUp(self):
        self.runner = CliRunner()
        self.cluster = "my-cluster"
        self.namespace = "kafka"
        self.topic = "my-topic"

    def test_no_option(self):
        result = self.runner.invoke(
            kfk,
            ["topics", "--topic", self.topic, "-c", self.cluster, "-n", self.namespace],
        )
        assert result.exit_code == 1
        assert "Missing options: kfk topics" in result.output

    @mock.patch("kfk.commands.topics.os")
    def test_list_topics(self, mock_os):
        result = self.runner.invoke(
            kfk, ["topics", "--list", "-c", self.cluster, "-n", self.namespace]
        )
        assert result.exit_code == 0
        mock_os.system.assert_called_with(
            Kubectl()
            .get()
            .kafkatopics()
            .label("strimzi.io/cluster={cluster}")
            .namespace(self.namespace)
            .build()
            .format(cluster=self.cluster)
        )

    @mock.patch("kfk.commands.topics.os")
    def test_describe_topic(self, mock_os):
        result = self.runner.invoke(
            kfk,
            [
                "topics",
                "--describe",
                "--topic",
                self.topic,
                "-c",
                self.cluster,
                "-n",
                self.namespace,
            ],
        )
        assert result.exit_code == 0
        mock_os.system.assert_called_with(
            Kubectl()
            .describe()
            .kafkatopics(self.topic)
            .namespace(self.namespace)
            .build()
        )

    @mock.patch("kfk.commands.topics.os")
    def test_describe_topic_output_yaml(self, mock_os):
        result = self.runner.invoke(
            kfk,
            [
                "topics",
                "--describe",
                "--topic",
                self.topic,
                "-c",
                self.cluster,
                "-n",
                self.namespace,
                "-o",
                "yaml",
            ],
        )
        assert result.exit_code == 0
        mock_os.system.assert_called_with(
            Kubectl()
            .get()
            .kafkatopics(self.topic)
            .namespace(self.namespace)
            .output("yaml")
            .build()
        )

    @mock.patch("kfk.commands.topics.os")
    def test_describe_topic_native(self, mock_os):
        result = self.runner.invoke(
            kfk,
            [
                "topics",
                "--describe",
                "--topic",
                self.topic,
                "-c",
                self.cluster,
                "-n",
                self.namespace,
                "--native",
            ],
        )
        assert result.exit_code == 0
        native_command = (
            "bin/kafka-topics.sh --bootstrap-server {cluster}-kafka-brokers:9092"
            " --describe --topic {topic}"
        )
        mock_os.system.assert_called_with(
            Kubectl()
            .exec("-it", "{cluster}-kafka-0")
            .container("kafka")
            .namespace(self.namespace)
            .exec_command(native_command)
            .build()
            .format(topic=self.topic, cluster=self.cluster)
        )

    @mock.patch("kfk.commons.transfer_file_to_container")
    @mock.patch("kfk.commands.topics.os")
    def test_describe_topic_native_with_command_config(
        self, mock_os, mock_transfer_file_to_container
    ):
        result = self.runner.invoke(
            kfk,
            [
                "topics",
                "--describe",
                "--topic",
                self.topic,
                "--command-config",
                "tests/files/client.properties",
                "-c",
                self.cluster,
                "-n",
                self.namespace,
                "--native",
            ],
        )
        assert result.exit_code == 0
        native_command = (
            "bin/kafka-topics.sh --bootstrap-server {cluster}-kafka-brokers:9093"
            " --describe --topic {topic} --command-config /tmp/client.properties;rm -rf"
            " /tmp/truststore.jks;rm -rf /tmp/user.p12;rm -rf /tmp/client.properties;"
        )
        mock_os.system.assert_called_with(
            Kubectl()
            .exec("-it", "{cluster}-kafka-0")
            .container("kafka")
            .namespace(self.namespace)
            .exec_command(native_command)
            .build()
            .format(topic=self.topic, cluster=self.cluster)
        )

    @mock.patch("kfk.commands.topics.create_temp_file")
    @mock.patch("kfk.commands.topics.create_using_yaml")
    def test_create_topic(self, mock_create_using_yaml, mock_create_temp_file):
        result = self.runner.invoke(
            kfk,
            [
                "topics",
                "--create",
                "--topic",
                self.topic,
                "--partitions",
                "12",
                "--replication-factor",
                "3",
                "-c",
                self.cluster,
                "-n",
                self.namespace,
            ],
        )

        assert result.exit_code == 0

        with open("tests/files/yaml/topic.yaml") as file:
            expected_topic_yaml = file.read()
            result_topic_yaml = mock_create_temp_file.call_args[0][0]
            assert expected_topic_yaml == result_topic_yaml

    @mock.patch("kfk.commands.topics.create_temp_file")
    @mock.patch("kfk.commands.topics.create_using_yaml")
    def test_create_topic_with_config(
        self, mock_create_using_yaml, mock_create_temp_file
    ):
        result = self.runner.invoke(
            kfk,
            [
                "topics",
                "--create",
                "--topic",
                self.topic,
                "--partitions",
                "24",
                "--replication-factor",
                "3",
                "--config",
                "min.insync.replicas=3",
                "-c",
                self.cluster,
                "-n",
                self.namespace,
            ],
        )

        assert result.exit_code == 0

        with open("tests/files/yaml/topic_with_one_config.yaml") as file:
            expected_topic_yaml = file.read()
            result_topic_yaml = mock_create_temp_file.call_args[0][0]
            assert expected_topic_yaml == result_topic_yaml

        mock_create_using_yaml.assert_called_once()

    def test_create_topic_without_required_params(self):
        result = self.runner.invoke(
            kfk,
            [
                "topics",
                "--create",
                "--topic",
                self.topic,
                "-c",
                self.cluster,
                "-n",
                self.namespace,
            ],
        )
        assert result.exit_code == 2

    @mock.patch("kfk.commands.topics.delete_using_yaml")
    def test_delete_topic(self, mock_delete_using_yaml):
        result = self.runner.invoke(
            kfk,
            [
                "topics",
                "--delete",
                "--topic",
                self.topic,
                "-c",
                self.cluster,
                "-n",
                self.namespace,
            ],
        )

        assert result.exit_code == 0

        mock_delete_using_yaml.assert_called_once()

    @mock.patch("kfk.commands.topics.create_temp_file")
    @mock.patch("kfk.commons.get_resource_yaml")
    @mock.patch("kfk.commands.topics.os")
    def test_alter_topic_with_no_params(
        self, mock_os, mock_get_resource_yaml, mock_create_temp_file
    ):
        with open("tests/files/yaml/topic.yaml") as file:
            expected_topic_yaml = file.read()
            mock_get_resource_yaml.return_value = expected_topic_yaml

            result = self.runner.invoke(
                kfk,
                [
                    "topics",
                    "--alter",
                    "--topic",
                    self.topic,
                    "-c",
                    self.cluster,
                    "-n",
                    self.namespace,
                ],
            )

            assert result.exit_code == 0

            result_topic_yaml = mock_create_temp_file.call_args[0][0]
            assert expected_topic_yaml == result_topic_yaml

    @mock.patch("kfk.commands.topics.create_temp_file")
    @mock.patch("kfk.commons.get_resource_yaml")
    @mock.patch("kfk.commands.topics.os")
    def test_alter_topic_without_config(
        self, mock_os, mock_get_resource_yaml, mock_create_temp_file
    ):
        with open("tests/files/yaml/topic.yaml") as file:
            topic_yaml = file.read()
            mock_get_resource_yaml.return_value = topic_yaml

            result = self.runner.invoke(
                kfk,
                [
                    "topics",
                    "--alter",
                    "--topic",
                    self.topic,
                    "--partitions",
                    "24",
                    "--replication-factor",
                    "3",
                    "-c",
                    self.cluster,
                    "-n",
                    self.namespace,
                ],
            )

            assert result.exit_code == 0

            with open("tests/files/yaml/topic_without_config.yaml") as file:
                expected_topic_yaml = file.read()
                result_topic_yaml = mock_create_temp_file.call_args[0][0]
                assert expected_topic_yaml == result_topic_yaml

    @mock.patch("kfk.commands.topics.create_temp_file")
    @mock.patch("kfk.commons.get_resource_yaml")
    @mock.patch("kfk.commands.topics.os")
    def test_alter_topic_with_config(
        self, mock_os, mock_get_resource_yaml, mock_create_temp_file
    ):
        with open("tests/files/yaml/topic.yaml") as file:
            topic_yaml = file.read()
            mock_get_resource_yaml.return_value = topic_yaml

            result = self.runner.invoke(
                kfk,
                [
                    "topics",
                    "--alter",
                    "--topic",
                    self.topic,
                    "--partitions",
                    "24",
                    "--replication-factor",
                    "3",
                    "--config",
                    "min.insync.replicas=3",
                    "-c",
                    self.cluster,
                    "-n",
                    self.namespace,
                ],
            )

            assert result.exit_code == 0

            with open("tests/files/yaml/topic_with_one_config.yaml") as file:
                expected_topic_yaml = file.read()
                result_topic_yaml = mock_create_temp_file.call_args[0][0]
                assert expected_topic_yaml == result_topic_yaml

    @mock.patch("kfk.commands.topics.create_temp_file")
    @mock.patch("kfk.commons.get_resource_yaml")
    @mock.patch("kfk.commands.topics.os")
    def test_alter_topic_with_two_configs(
        self, mock_os, mock_get_resource_yaml, mock_create_temp_file
    ):
        with open("tests/files/yaml/topic.yaml") as file:
            topic_yaml = file.read()
            mock_get_resource_yaml.return_value = topic_yaml

            result = self.runner.invoke(
                kfk,
                [
                    "topics",
                    "--alter",
                    "--topic",
                    self.topic,
                    "--partitions",
                    "24",
                    "--replication-factor",
                    "3",
                    "--config",
                    "min.insync.replicas=3",
                    "--config",
                    "cleanup.policy=compact",
                    "-c",
                    self.cluster,
                    "-n",
                    self.namespace,
                ],
            )

            assert result.exit_code == 0

            with open("tests/files/yaml/topic_with_two_configs.yaml") as file:
                expected_topic_yaml = file.read()
                result_topic_yaml = mock_create_temp_file.call_args[0][0]
                assert expected_topic_yaml == result_topic_yaml

    @mock.patch("kfk.commands.topics.create_temp_file")
    @mock.patch("kfk.commons.get_resource_yaml")
    @mock.patch("kfk.commands.topics.os")
    def test_alter_topic_with_two_configs_delete_one_config(
        self, mock_os, mock_get_resource_yaml, mock_create_temp_file
    ):
        with open("tests/files/yaml/topic_with_two_configs.yaml") as file:
            topic_yaml = file.read()
            mock_get_resource_yaml.return_value = topic_yaml

            result = self.runner.invoke(
                kfk,
                [
                    "topics",
                    "--alter",
                    "--topic",
                    self.topic,
                    "--delete-config",
                    "cleanup.policy",
                    "-c",
                    self.cluster,
                    "-n",
                    self.namespace,
                ],
            )

            assert result.exit_code == 0

            with open("tests/files/yaml/topic_with_one_config.yaml") as file:
                expected_topic_yaml = file.read()
                result_topic_yaml = mock_create_temp_file.call_args[0][0]
                assert expected_topic_yaml == result_topic_yaml

    @mock.patch("kfk.commands.topics.create_temp_file")
    @mock.patch("kfk.commons.get_resource_yaml")
    @mock.patch("kfk.commands.topics.os")
    def test_alter_topic_with_two_configs_delete_two_configs(
        self, mock_os, mock_get_resource_yaml, mock_create_temp_file
    ):
        with open("tests/files/yaml/topic_with_two_configs.yaml") as file:
            topic_yaml = file.read()
            mock_get_resource_yaml.return_value = topic_yaml

            result = self.runner.invoke(
                kfk,
                [
                    "topics",
                    "--alter",
                    "--topic",
                    self.topic,
                    "--delete-config",
                    "cleanup.policy",
                    "--delete-config",
                    "min.insync.replicas",
                    "-c",
                    self.cluster,
                    "-n",
                    self.namespace,
                ],
            )

            assert result.exit_code == 0

            with open("tests/files/yaml/topic_without_config.yaml") as file:
                expected_topic_yaml = file.read()
                result_topic_yaml = mock_create_temp_file.call_args[0][0]
                assert expected_topic_yaml == result_topic_yaml



import os

import click
import yaml

from kfk.commands import clusters, topics, users
from kfk.commands.main import kfk
from kfk.commons import (
    get_config_list,
    get_list_by_split_string,
    get_resource_as_stream,
    raise_exception_for_missing_options,
)
from kfk.constants import COMMON_NAME_PREFIX, KAFKA_PORT, NEW_LINE, SPACE, SpecialTexts
from kfk.kubectl_command_builder import Kubectl
from kfk.messages import Messages
from kfk.option_extensions import NotRequiredIf


@click.option("-n", "--namespace", help="Namespace to use", required=True)
@click.option("-c", "--cluster", help="Cluster to use", required=True)
@click.option("--delete-config", help="Config keys to remove")
@click.option("--add-config", help="Key Value pairs of configs to add.")
@click.option("--alter", help="Alter the configuration for the entity.", is_flag=True)
@click.option(
    "--native", help="List configs for the given entity natively.", is_flag=True
)
@click.option("--describe", help="List configs for the given entity.", is_flag=True)
@click.option(
    "--entity-name",
    help="Name of entity",
    required=True,
    cls=NotRequiredIf,
    options=["native"],
)
@click.option(
    "--entity-type",
    help="Type of entity (topics/users/brokers)",
    type=click.Choice(["topics", "users", "brokers"], case_sensitive=True),
)
@kfk.command()
def configs(
    entity_type,
    entity_name,
    describe,
    native,
    alter,
    add_config,
    delete_config,
    cluster,
    namespace,
):
    """Adds/Removes entity config for a topic, client, user or brokers."""
    if describe:
        if entity_type == "topics":
            if native:
                _describe_natively(entity_type, entity_name, cluster, namespace)
            else:
                topics.describe(entity_name, None, False, None, cluster, namespace)
        elif entity_type == "users":
            if native:
                _describe_natively(entity_type, entity_name, cluster, namespace)
            else:
                users.describe(entity_name, None, cluster, namespace)
        elif entity_type == "brokers":
            if native:
                _describe_natively(entity_type, entity_name, cluster, namespace)
                stream = get_resource_as_stream(
                    "configmap",
                    resource_name=cluster + "-kafka-config",
                    namespace=namespace,
                )
                config_dict = yaml.full_load(stream)
                config_data = get_list_by_split_string(
                    config_dict["data"]["server.config"],
                    SpecialTexts.BROKER_CONFIG_FILE_USER_CONFIG_HEADER,
                )[1]
                click.echo(
                    NEW_LINE
                    + Messages.USER_PROVIDED_CONFIG_HEADER
                    + NEW_LINE
                    + config_data
                )
            else:
                clusters.describe(cluster, None, namespace)

    elif alter:
        add_config_list = get_config_list(add_config)
        delete_config_list = get_config_list(delete_config)

        if entity_type == "topics":
            topics.alter(
                entity_name,
                None,
                None,
                add_config_list,
                delete_config_list,
                cluster,
                namespace,
            )
        elif entity_type == "users":
            users.alter(
                entity_name,
                None,
                None,
                False,
                False,
                tuple(),
                None,
                None,
                None,
                None,
                None,
                add_config_list,
                delete_config_list,
                cluster,
                namespace,
            )
        elif entity_type == "brokers":
            clusters.alter(
                entity_name, None, None, add_config_list, delete_config_list, namespace
            )
    else:
        raise_exception_for_missing_options("configs")


def _describe_natively(entity_type, entity_name, cluster, namespace):
    native_command = (
        "bin/kafka-configs.sh --bootstrap-server {cluster}-kafka-brokers:{port} "
        "--entity-type {entity_type} --describe"
    )

    if entity_name is not None:
        native_command = native_command + SPACE + "--entity-name {entity_name}"

        if entity_type == "users":
            entity_name = COMMON_NAME_PREFIX + entity_name

    os.system(
        Kubectl()
        .exec("-it", cluster + "-kafka-0")
        .container("kafka")
        .namespace(namespace)
        .exec_command(native_command)
        .build()
        .format(
            cluster=cluster,
            port=KAFKA_PORT,
            entity_type=entity_type,
            entity_name=entity_name,
        )
    )



import click


class NotRequiredIf(click.Argument):
    # TODO: Refactor here

    def __init__(self, *args, **kwargs):
        self.arguments = kwargs.pop("arguments")
        assert self.arguments, "'arguments' parameter required"
        super(NotRequiredIf, self).__init__(*args, **kwargs)

    def handle_parse_result(self, ctx, opts, args):
        control_arguments_exist = False

        for arguments in self.arguments:
            control_arguments_exist = arguments in opts
            if control_arguments_exist is True:
                break

        if control_arguments_exist:
            self.required = None

        return super(NotRequiredIf, self).handle_parse_result(ctx, opts, args)


class RequiredIf(click.Argument):
    # TODO: Refactor here

    def __init__(self, *args, **kwargs):
        self.arguments = kwargs.pop("arguments")
        assert self.arguments, "'arguments' parameter required"
        super(RequiredIf, self).__init__(*args, **kwargs)

    def handle_parse_result(self, ctx, opts, args):
        control_arguments_exist = False

        for arguments in self.arguments:
            control_arguments_exist = arguments in opts
            if control_arguments_exist is True:
                break

        if control_arguments_exist:
            self.required = True
        else:
            self.required = None

        return super(RequiredIf, self).handle_parse_result(ctx, opts, args)
